/*
 * Creaci de la taula APLICACIO.
 * {aplicacio} varchar que indica el nom de l'aplicaci (CLAU PRIMARIA).
 */

CREATE TABLE aplicacio (
     aplicacio VARCHAR2(50) NOT NULL,
     CONSTRAINT aplicacio_pk PRIMARY KEY (aplicacio)
);

/*
 * Creaci de la taula VERSIO.
 * {versio} varchar que indica la versi de la que es tracta (CLAU PRIMARIA).
 */

CREATE TABLE versio (
     versio VARCHAR2(20) NOT NULL,
     CONSTRAINT versio_pk PRIMARY KEY (versio)
);

/*
 * Creaci de la taula PAIS.
 * {identificador} varchar que identifica el pas mitjanant la ISO 3166-1 
 * alfa-2 (CLAU PRIMARIA).
 * {pais} varchar que indica el nom del pas.
 */

CREATE TABLE pais (
     identificador VARCHAR2(2) NOT NULL,
     pais VARCHAR2(50) UNIQUE NOT NULL,
     CONSTRAINT pais_pk PRIMARY KEY (identificador)
);

/*
 * Creaci de la taula IDIOMAS.
 * {identificador} varchar que identifica l'idioma mitjanant la ISO 639 (CLAU 
 * PRIMARIA).
 * {idioma} varchar que indica el nom de l'idioma.
 */

CREATE TABLE idioma (
     identificador VARCHAR2(2) NOT NULL,
     idioma VARCHAR2(50) UNIQUE NOT NULL,
     CONSTRAINT idioma_pk PRIMARY KEY (identificador)
);

/*
 * Creaci de la taula PERSONA.
 * {identificador} varchar que indica el DNI de la persona (CLAU PRIMARIA).
 * {nom} varchar que indica el nom de la persona.
 * {adreca} varchar que indica l'adrea de la persona.
 * {telefon} nmero que indica el telefon de la persona.
 * {mail} varchar que indica la adrea de correo electrnic de la persona.
 * {activo} varchar per a emular un booleano 'false'=FALSE 'true'=TRUE.
 * {pais} varchar que identifica el pas (CLAU FORANA cap a pais(identificador))
 */
 
CREATE TABLE persona (
     identificador VARCHAR2(9) NOT NULL,
     nom VARCHAR2(50) NOT NULL,
     adreca VARCHAR2(100) NOT NULL,
	   telefon NUMBER(9) UNIQUE NOT NULL,
	   mail VARCHAR2(50) UNIQUE NOT NULL,
     activo VARCHAR2(5) DEFAULT 'true' NOT NULL,
	   pais VARCHAR2(2) NOT NULL,
     CONSTRAINT persona_pk PRIMARY KEY (identificador),
     CONSTRAINT persona_pais_fk FOREIGN KEY (pais) REFERENCES pais(identificador),
     CONSTRAINT booleano_persona_activo_ck CHECK (activo in ('true', 'false'))
);

/*
 * Creaci de la taula USUARI.
 * {identificador} varchar que indica el DNI de l'usuari (CLAU PRIMARIA) i (CLAU
 * FORANA cap a persona(identificador)).
 * {operador} varchar que identifica l'operador amb el qui treballa l'usuari 
 * {primera_descarrega} varchar per a emular un booleano 'false'=FALSE 
 * 'true'=TRUE.
 */
 
CREATE TABLE usuari (
     identificador VARCHAR2(9) NOT NULL,
     operador VARCHAR(50) NOT NULL,
     primera_descarrega VARCHAR2(5) DEFAULT 'false' NOT NULL,
     CONSTRAINT usuari_pk PRIMARY KEY (identificador),
     CONSTRAINT usuari_persona_fk FOREIGN KEY (identificador) REFERENCES persona(identificador),
     CONSTRAINT booleano_primera_descarrega_ck CHECK (primera_descarrega in ('true', 'false'))
);

/*
 * Creaci de la taula DESENVOLUPADOR.
 * {identificador} varchar que indica el DNI de l'usuari (CLAU PRIMARIA) i (CLAU
 * FORANA cap a persona(identificador)).
 * {representant} varchar que identifica el nom del representant legal de 
 * l'empresa.
 * {descarregues_app_propies} number per emmagatzemar la quantitat de 
 * descarregues d'aplicacions desenvolupades per el desenvolupador.
 */
 
CREATE TABLE desenvolupador (
     identificador VARCHAR2(9) NOT NULL,
     representant VARCHAR2(50) NOT NULL,
     descarregues_app_propies NUMBER(10) DEFAULT 0 NOT NULL,
     CONSTRAINT desenvolupador_pk PRIMARY KEY (identificador),
     CONSTRAINT desenvolupador_persona_fk FOREIGN KEY (identificador) REFERENCES persona(identificador)
);

/*
 * Creaci de la taula SSOO
 * {identificador} number amb un mxim de 5 dgits auto incremental mitjanant  
 * la seqncia seq_id_ssoo i el trigger trig_id_ssoo (CLAU PRIMARIA).
 * {ssoo} varchar que indica el nom del ssoo.
 */

CREATE TABLE ssoo (
     identificador NUMBER(5) NOT NULL,
     ssoo VARCHAR2(50) UNIQUE NOT NULL,
     CONSTRAINT ssoo_pk PRIMARY KEY (identificador)
);

/*
 * Seqncia creada para la realitzaci de un identificador auto incremental per
 * la taula SSOO.
 * Comenament en 1 increment +1 sense valor mxim.
 */
 
CREATE SEQUENCE seq_id_ssoo
START WITH 1
INCREMENT BY 1
NOMAXVALUE;

/*
 * Trigger necessari per la creaci d'un identificador auto incremental en la 
 * taula SSOO.
 * Tipus BEFORE INSERT - FOR EACH ROW
 */
 
CREATE OR REPLACE TRIGGER trig_id_ssoo
BEFORE INSERT ON ssoo
FOR EACH ROW
BEGIN
SELECT seq_id_ssoo.nextval INTO :new.identificador FROM DUAL;
END trig_id_ssoo;
/

/*
 * Creaci de la taula RESOLUCIO.
 * {identificador} number amb un mxim de 5 dgits auto incremental mitjanant  
 * la seqncia seq_id_dispositiu i el trigger trig_id_dispositiu (CLAU PRIMARIA)
 * {resolucio} varchar que indica la resoluci.
 */

CREATE TABLE resolucio (
     identificador NUMBER(5) NOT NULL,
     resolucio VARCHAR2(20) UNIQUE NOT NULL,
     CONSTRAINT resolucio_pk PRIMARY KEY (identificador)
);

/*
 * Seqncia creada para la realitzaci de un identificador auto incremental per
 * la taula RESOLUCIO.
 * Comenament en 1 increment +1 sense valor mxim.
 */
 
CREATE SEQUENCE seq_id_resolucio
START WITH 1
INCREMENT BY 1
NOMAXVALUE;

/*
 * Trigger necessari per la creaci d'un identificador auto incremental en la 
 * taula RESOLUCIO
 * Tipus BEFORE INSERT - FOR EACH ROW
 */
 
CREATE OR REPLACE TRIGGER trig_id_resolucio
BEFORE INSERT ON resolucio
FOR EACH ROW
BEGIN
SELECT seq_id_resolucio.nextval INTO :new.identificador FROM DUAL;
END trig_id_resolucio;
/

/*
 * Creaci de la taula DISPOSITIU.
 * {marca} varchar que indica la marca del dispositiu.
 * {model} varchar que indica el model del dispositiu.
 * {marca, model} CLAU PRIMARIA.
 * {familia} varchar que indica la familia del dispositiu.
 * {ssoo} number que identifica el ssoo (CLAU FORANA cap a ssoo(identificador)).
 * {resolucio} number que identifica la resoluci del dispositiu (CLAU FORANA 
 * cap a resolucio(identificador)).
 */

CREATE TABLE dispositiu (
     marca VARCHAR(50) NOT NULL,
     model VARCHAR(50) NOT NULL,
     familia VARCHAR(50) NOT NULL,
     ssoo NUMBER(5) NOT NULL,
     resolucio NUMBER(5) NOT NULL,
     CONSTRAINT dispositiu_pk PRIMARY KEY (marca, model),
     CONSTRAINT dispositiu_ssoo_fk FOREIGN KEY (ssoo) REFERENCES ssoo(identificador),
     CONSTRAINT dispositiu_resolucio_fk FOREIGN KEY (resolucio) REFERENCES resolucio(identificador)
);

/*
 * Creaci de la taula PAGAMENT
 * {pagament} varchar que indica el modo de pagament (CLAU PRIMARIA).
 */

CREATE TABLE pagament (
     pagament VARCHAR2(50) NOT NULL,
     CONSTRAINT pagament_pk PRIMARY KEY (pagament)
);

/*
 * Creaci de la taula APLICACIO_VERSIONADA.
 * {aplicacio} varchar que identifica l'aplicaci (CLAU FORANA cap a 
 * aplicacio(identificador)).
 * {versio} varchar que identifica la versi (CLAU FORANA cap a 
 * versio(identificador)).
 * {resolucio_minima} number que identifica la resoluci necessria (CLAU FORANA
 * cap a resolucio(identificador)).
 * {url} varchar amb la url on es troba el vdeo de demostraci (admet valors 
 * NULL).
 * {data_pujada} date que representa la data en la que es va donar d'alta 
 * l'aplicaci en el sistema.
 * {activa} varchar per a emular un booleano 'false'=FALSE 'true'=TRUE
 * {euros_generats} number que acumula els ingresos generats per la descarrega
 * de l'aplicaci.
 * {paisos_descarregada} varchar que recull una lista de codis de paisos on s'ha
 * descarregat l'aplicaci (admet valors NULL).
 * {aplicacio, versio} CLAU PRIMARIA.
 */

CREATE TABLE aplicacio_versionada (
     aplicacio VARCHAR(50) NOT NULL,
     versio VARCHAR(20) NOT NULL,
     resolucio_minima NUMBER(5) NOT NULL,
     url VARCHAR2(100),
     data_pujada DATE NOT NULL,
     activa VARCHAR2(5) DEFAULT 'true' NOT NULL,
     euros_generats NUMBER (10,2) DEFAULT 0 NOT NULL,
     paisos_descarregada VARCHAR2(50),
     CONSTRAINT aplicacio_versionada_pk PRIMARY KEY (aplicacio, versio),
     CONSTRAINT aplicacio_versionada_aplic_fk FOREIGN KEY (aplicacio) REFERENCES aplicacio(aplicacio),
     CONSTRAINT aplicacio_versionada_versio_fk FOREIGN KEY (versio) REFERENCES versio(versio),
     CONSTRAINT aplicacio_versionada_resol_fk FOREIGN KEY (resolucio_minima) REFERENCES resolucio(identificador),
     CONSTRAINT booleano_aplic_activa_ck CHECK (activa in ('true', 'false'))        
);

/*
 * Creaci de la taula PREU.
 * {aplicacio} varchar que identifica l'aplicaci.
 * {versio} varchar que identifica la versi.
 * {aplicacio, versio} identifica l'aplicaci versionada (CLAU FORANA cap a 
 * aplicacio_versionada(aplicacio, versio)).
 * {pais} varchar que identifica el pas (CLAU FORANA cap a pais(identificador))
 * {preu} NUMBER que indica el preu en euros.
 * {aplicacio, versio, pais} CLAU PRIMARIA.
 */

CREATE TABLE preu (
     aplicacio VARCHAR(50) NOT NULL,
     versio VARCHAR(20) NOT NULL,
     pais VARCHAR(2) NOT NULL,
     preu NUMBER(6,2) NOT NULL,
     CONSTRAINT preu_pk PRIMARY KEY (aplicacio, versio, pais),
     CONSTRAINT preu_aplicacio_versionada_fk FOREIGN KEY (aplicacio, versio) REFERENCES aplicacio_versionada(aplicacio, versio),
     CONSTRAINT preu_pais_fk FOREIGN KEY (pais) REFERENCES pais(identificador),
     CONSTRAINT preu_positiu_ck CHECK (preu>=0)
);

/*
 * Creaci de la taula DESCRIPCIO.
 * {aplicacio} varchar que identifica l'aplicaci.
 * {versio} varchar que identifica la versi.
 * {aplicacio, versio} identifica l'aplicaci versionada (CLAU FORANA cap a 
 * aplicacio_versionada(aplicacio, versio)).
 * {idioma} varchar que identifica l'idioma (CLAU FORANA cap a 
 * idioma(identificador)).
 * {descripcio} varchar que contindr la descripci completa.
 * {aplicacio, versio, idioma} CLAU PRIMARIA.
 */

CREATE TABLE descripcio (
     aplicacio VARCHAR(50) NOT NULL,
     versio VARCHAR(20) NOT NULL,
     idioma VARCHAR(2) NOT NULL,
     descripcio VARCHAR2(4000) NOT NULL,
     CONSTRAINT descripcio_pk PRIMARY KEY (aplicacio, versio, idioma),
     CONSTRAINT descripcio_aplic_versionada_fk FOREIGN KEY (aplicacio, versio) REFERENCES aplicacio_versionada(aplicacio, versio),
     CONSTRAINT descripcio_idioma_fk FOREIGN KEY (idioma) REFERENCES idioma(identificador)
);

/*
 * Creaci de la taula SSOO_APLICACIO.
 * {aplicacio} varchar que identifica l'aplicaci.
 * {versio} varchar que identifica la versi.
 * {aplicacio, versio} identifica l'aplicaci versionada (CLAU FORANA cap a 
 * aplicacio_versionada(aplicacio, versio)).
 * {ssoo} number que identifica el ssoo (CLAU FORANA cap a ssoo(identificador)).
 * {binari} varchar que contindr l'enlla a l'arxiu binari.
 * {mida} number que indicar la mida en MB.
 * {aplicacio, versio, ssoo} CLAU PRIMARIA.
 */

CREATE TABLE ssoo_aplicacio (
     aplicacio VARCHAR(50) NOT NULL,
     versio VARCHAR(20) NOT NULL,
     ssoo NUMBER(5) NOT NULL,
     binari VARCHAR2(100) NOT NULL,
     mida NUMBER(5,2) NOT NULL,
     CONSTRAINT ssoo_aplicacio_pk PRIMARY KEY (aplicacio, versio, ssoo),
     CONSTRAINT ssoo_aplic_aplic_versionada_fk FOREIGN KEY (aplicacio, versio) REFERENCES aplicacio_versionada(aplicacio, versio),
     CONSTRAINT ssoo_aplic_ssoo_fk FOREIGN KEY (ssoo) REFERENCES ssoo(identificador),
     CONSTRAINT mida_positiva_ck CHECK (mida>0)
);

/*
 * Creaci de la taula DESENVOLUPADOR_APLICACIO.
 * {aplicacio} varchar que identifica l'aplicaci.
 * {versio} varchar que identifica la versi.
 * {aplicacio, versio} identifica l'aplicaci versionada (CLAU FORANA cap a 
 * aplicacio_versionada(aplicacio, versio)).
 * {desenvolupador} varchar que identifica el desenvolupador (CLAU FORANA cap a 
 * desenvolupador(identificador)).
 * {aplicacio, versio, desenvolupador} CLAU PRIMARIA.
 */

CREATE TABLE desenvolupador_aplicacio (
     aplicacio VARCHAR(50) NOT NULL,
     versio VARCHAR(20) NOT NULL,
     desenvolupador VARCHAR2(9) NOT NULL,
     CONSTRAINT desenvolupador_aplicacio_pk PRIMARY KEY (aplicacio, versio, desenvolupador),
     CONSTRAINT desenv_aplic_aplic_vers_fk FOREIGN KEY (aplicacio, versio) REFERENCES aplicacio_versionada(aplicacio, versio),
     CONSTRAINT desenv_aplic_desenv_fk FOREIGN KEY (desenvolupador) REFERENCES desenvolupador(identificador)
);

/*
 * Creaci de la taula DISPOSITIU_USUARI.
 * {imei} number que indica l'imei del dispositiu (CLAU PRIMARIA).
 * {usuari} varchar que identifica l'usuari (CLAU FORANA cap a 
 * usuari(identificador)).
 * {marca} varchar que identifica el dispositiu amb la seva marca.
 * {model} varchar que identifica el dispositiu amb el seu model.
 * {marca, model} CLAU FORANA cap a dispositiu(marca, model).
 */

CREATE TABLE dispositiu_usuari (
     imei NUMBER(15) NOT NULL,
     usuari VARCHAR2(9) NOT NULL,
     marca  VARCHAR(50) NOT NULL,    
     model  VARCHAR(50) NOT NULL,
     CONSTRAINT dispositiu_usuari_pk PRIMARY KEY (imei),
     CONSTRAINT dispositiu_usuari_user_fk FOREIGN KEY (usuari) REFERENCES usuari(identificador),
     CONSTRAINT dispositiu_usuari_disp_fk FOREIGN KEY (marca, model) REFERENCES dispositiu(marca, model)
);

/*
 * Creaci de la taula DESCARREGA.
 * {identificador} number amb un mxim de 20 dgits auto incremental mitjanant
 * la seqncia seq_id_descarrega i el trigger trig_id_descarrega(CLAU PRIMARIA).
 * {data} date que representa la data en la que es va realitzar la descarrega.
 * {aplicacio} varchar que identifica l'aplicaci.
 * {versio} varchar que identifica la versi.
 * {aplicacio, versio} identifica l'aplicaci versionada (CLAU FORANA cap a 
 * aplicacio_versionada(aplicacio, versio)).
 * {dispositiu} number que identifica el dispositiu_usuari on s'ha realitzat la 
 * descarrega (CLAU FORANA cap a dispositiu_usuari(imei)).
 * {pagament} varchar que identifica el pagament (CLAU FORANA cap a 
 * pagament(pagament))
 * {preu} number que indica el preu en euros pagat per la descarrega de 
 * l'aplicaci.
 */

CREATE TABLE descarrega (
     identificador NUMBER(20) NOT NULL,
     data DATE NOT NULL,
     aplicacio VARCHAR(50) NOT NULL,
     versio VARCHAR(20) NOT NULL,    
     dispositiu  NUMBER(15) NOT NULL,
     pagament VARCHAR(50) NOT NULL,
     preu NUMBER(6,2) NOT NULL,
     CONSTRAINT descarrega_pk PRIMARY KEY (identificador),
     CONSTRAINT descarrega_aplicacio_ver_fk FOREIGN KEY (aplicacio, versio) REFERENCES aplicacio_versionada(aplicacio, versio),
     CONSTRAINT descarrega_dispositiu_fk FOREIGN KEY (dispositiu) REFERENCES dispositiu_usuari(imei),
     CONSTRAINT descarrega_pagament_fk FOREIGN KEY (pagament) REFERENCES pagament(pagament),
     CONSTRAINT preu_descarrega_ck CHECK (preu>=0)
);

/*
 * Seqncia creada para la realitzaci de un identificador auto incremental per
 * la taula DESCARREGA.
 * Comenament en 1 increment +1 sense valor mxim.
 */
 
CREATE SEQUENCE seq_id_descarrega
START WITH 1
INCREMENT BY 1
NOMAXVALUE;

/*
 * Trigger necessari per la creaci d'un identificador auto incremental en la 
 * taula DESCARREGA.
 * Tipus BEFORE INSERT - FOR EACH ROW
 */
 
CREATE OR REPLACE TRIGGER trig_id_descarrega
BEFORE INSERT ON descarrega
FOR EACH ROW
BEGIN
SELECT seq_id_descarrega.nextval INTO :new.identificador FROM DUAL;
END trig_id_descarrega;
/

/*
 * Creaci de la taula LOG
 * {identificador} number amb un mxim de 10 dgits auto incremental mitjanant
 * la seqncia seq_id_log i el trigger trig_id_log (CLAU PRIMARIA).
 * {data_hora} varchar que representa la data i hora en la que es va realitzar 
 * la crida.
 * {procediment} varchar que indica el nom del procediment que ha realitzat la 
 * crida.
 * {param_entrada} varchar que indica el valor dels parametres d'entrada.
 * {param_sortida} varchar que indica el resultat de la crida amb el seu valor 
 * de sortida.
 */
 
CREATE TABLE log (
  identificador NUMBER(20) NOT NULL,
  data_hora VARCHAR2(20) NOT NULL,
  procediment VARCHAR2(50) NOT NULL,
  param_entrada VARCHAR2(200) NOT NULL,
  param_sortida	VARCHAR2(200) NOT NULL, 
  CONSTRAINT log_pk PRIMARY KEY (identificador)
);

/*
 * Seqncia creada para la realitzaci de un identificador auto incremental per
 * la taula LOG.
 * Comenament en 1 increment +1 sense valor mxim.
 */
 
CREATE SEQUENCE seq_id_log
START WITH 1
INCREMENT BY 1
NOMAXVALUE;

/*
 * Trigger necessari per la creaci d'un identificador auto incremental en la 
 * taula LOG.
 * Tipus BEFORE INSERT - FOR EACH ROW
 */
 
CREATE OR REPLACE TRIGGER trig_id_log
BEFORE INSERT ON log
FOR EACH ROW
BEGIN
SELECT seq_id_log.nextval INTO :new.identificador FROM DUAL;
END trig_id_log;
/

/*
 * Creaci de la taula EST1_2_TOTALS
 * {total_descarregues} number amb un mxim de 10 dgits que emmagatzemar el
 * total de descarregues generades fins al moment.
 * {total_euros} number amb un mxim de 15 dgits que emmagatzemar el
 * total de euros generats fins al moment.
 * {total_usuaris} number amb un mxim de 10 dgits que emmagatzemar el
 * total d'usuaris registrats en el sistema.
 * {total_descarregue, total_euros, total_usuaris} CLAU PRIMARIA
 */
 
CREATE TABLE est1_2_totals (
  total_descarregues NUMBER(10) NOT NULL,
  total_euros NUMBER(15,2) NOT NULL,
  total_usuaris NUMBER(10) NOT NULL,
  CONSTRAINT est1_2_pk PRIMARY KEY (total_descarregues, total_euros, total_usuaris),
  CONSTRAINT total_desc_est1_ck CHECK (total_descarregues>=0),
  CONSTRAINT total_euros_est2_ck CHECK (total_euros>=0),
  CONSTRAINT total_users_est2_ck CHECK (total_usuaris>=0)
);

/*
 * Creaci de la taula EST3_4_5_ANY.
 * {any_consultat} number amb un mxim de 4 dgits que indica l'any (CLAU 
 * PRIMARIA)
 * {descarregues} number que indica les descarregues totals de l'any
 * {usuaris} number que indica la quantitat total d'usuaris del sistema durant 
 * l'any.
 * {mitjana_desc_user} number que indicar la mitjana de descarregues per 
 * usuari i any.
 * {nom_best_desenv} varchar amb el nom del desenvolupador amb ms descarregues.
 * {desc_best_desenv} number que indica la quantitat de descarregues 
 * d'aplicacions creades pel desenvolupador amb ms descarregues de l'any.
 * {best_aplicacio} varchar amb el nom de l'aplicaci que ha generat ms 
 * ingressos durant l'any.
 * {euros_best_aplicacio} number amb la quantitat total d'ingresos generats per 
 * l'aplicaci que ms a recaudat durant l'any.
 * {desenv_best_aplicacio} varchar amb el nom del/s desenvolupador/s que han 
 * creat l'aplicaci que ms ha recaudat durant l'any.
 */
 
CREATE TABLE est3_4_5_any (
  any_consultat NUMBER(4) NOT NULL,
  descarregues NUMBER(10) DEFAULT 0 NOT NULL,
  usuaris NUMBER(10) NOT NULL,
  mitjana_desc_user number(5,2) DEFAULT 0 NOT NULL,
  nom_best_desenv VARCHAR2(200) NOT NULL,
  desc_best_desenv NUMBER(10) DEFAULT 0 NOT NULL,
  best_aplicacio VARCHAR2(50) NOT NULL,
  euros_best_aplicacio NUMBER(10,2) DEFAULT 0 NOT NULL,
  desenv_best_aplicacio VARCHAR2(200) NOT NULL,  
  CONSTRAINT est3_4_5_pk PRIMARY KEY (any_consultat),
  CONSTRAINT any_est3_4_5_ck CHECK (any_consultat>=2012),
  CONSTRAINT total_descarregues_est3_ck CHECK (descarregues>=0),
  CONSTRAINT total_usuaris_est3_ck CHECK (usuaris>=0),
  CONSTRAINT mitjana_est3_ck CHECK (mitjana_desc_user>=0)
);

/*
 * Creaci de la taula EST6_7_8_ANY_PAIS.
 * {any_consultat} number amb un mxim de 4 dgits que indica l'any.
 * {pais} varchar amb el nom del pas.
 * {any_consultat, pais} (CLAU PRIMARIA).
 * {usuaris_1_desc} number amb la quantitat d'usuaris que han fet al menys una 
 * descarrega en el pas i any donat.
 * {aplicacions_1_desc} number amb la quantitat total d'aplicacions que s'han 
 * descarregat al menys una vegada en el pas i any donat.
 * {total_euros} number amb la quantitat total d'ingressos generats en el pas i
 * any donat.
 */
 
CREATE TABLE est6_7_8_any_pais (
  any_consultat NUMBER(4) NOT NULL,
  pais VARCHAR2(50) NOT NULL,
  usuaris_1_desc NUMBER(10) DEFAULT 0 NOT NULL,
  aplicacions_1_desc NUMBER(10) DEFAULT 0 NOT NULL,
  total_euros NUMBER(10,2) DEFAULT 0 NOT NULL,
  CONSTRAINT est6_7_8_pk PRIMARY KEY (any_consultat, pais),
  CONSTRAINT any_est6_7_8_ck CHECK (any_consultat>=2012),
  CONSTRAINT usuaris_est6_ck CHECK (usuaris_1_desc>=0),
  CONSTRAINT total_euros_est7_ck CHECK (total_euros>=0),
  CONSTRAINT aplicacions_est8_ck CHECK (aplicacions_1_desc>=0)  
);