
/*
 * METODES ABM (ALTA, BAIXA I MODIFICACIO) I ALTRES METODES DE PROPOSIT GENERAL
 */

/*
 * Mtode que afegeix un registre al LOG.
 * {pProcediment} varchar amb el nom del procediment executat.
 * {pEntrada} varchar amb els noms dels parmetres d'entrada i els seus valors.
 * {pSortida} varchar amb el resultat de la execuci del mtode.
 */
CREATE OR REPLACE PROCEDURE afegirLog(pProcediment IN VARCHAR2, 
                            pEntrada IN VARCHAR2, pSortida IN VARCHAR2) IS

diaHora VARCHAR2(20);
PRAGMA AUTONOMOUS_TRANSACTION;

BEGIN
  
  --Obt la data i hora actual del sistema.
  SELECT TO_CHAR(SYSDATE, 'DD/MM/YY, HH24:MI:SS') INTO diaHora FROM DUAL;
  
  INSERT INTO log(data_hora, procediment, param_entrada, param_sortida) 
  VALUES (diaHora, pProcediment, pEntrada, pSortida);
  
  COMMIT;
                
END afegirLog;
/


/*
 * Mtode que verifica l'existncia d'un registre en la taula Persona.
 * {pDni} varchar amb l'identificador de la persona.
 * {pExisteix} number que indica si existeix o no el registre a la taula.
 * 1 --> existeix, 0 --> NO existeix.
 */
CREATE OR REPLACE PROCEDURE existeixPersona(pDni IN VARCHAR2, 
                                            pExisteix OUT NUMBER) IS

BEGIN
  
  SELECT COUNT(*)
  INTO pExisteix
  FROM persona p
  WHERE p.identificador = pDni;
                
END existeixPersona;
/


/*
 * Mtode que verifica si una persona s un Usuari.
 * {pDni} varchar amb l'identificador de la persona.
 * {pEs} number que indica si la persona s un usuari.
 * 1 --> s un usuari, 0 --> NO s un usuari.
 */
CREATE OR REPLACE PROCEDURE esUsuari(pDni IN VARCHAR2, pEs OUT NUMBER) IS

BEGIN
  
  SELECT COUNT(*)
  INTO pEs
  FROM usuari u
  WHERE u.identificador = pDni;
                
END esUsuari;
/


/*
 * Mtode que verifica si una persona s un Desenvolupador.
 * {pDni} varchar amb l'identificador de la persona.
 * {pEs} number que indica si la persona s un desenvolupador.
 * 1 --> s un desenvolupador, 0 --> NO s un desenvolupador.
 */
CREATE OR REPLACE PROCEDURE esDesenvolupador(pDni IN VARCHAR2, 
                                             pEs OUT NUMBER) IS

BEGIN
  
  SELECT COUNT(*)
  INTO pEs
  FROM desenvolupador d
  WHERE d.identificador = pDni;
                
END esDesenvolupador;
/


/*
 * Mtode que verifica l'existncia d'un registre en la taula Dispositiu.
 * {pMarca} varchar amb la marca del dispositiu.
 * {pModel} varchar amb el model del dispositiu.
 * {pExisteix} number que indica si existeix o no el registre a la taula.
 * 1 --> existeix, 0 --> NO existeix.
 */
CREATE OR REPLACE PROCEDURE existeixDispositiu(pMarca IN VARCHAR2, 
                            pModel IN VARCHAR2, pExisteix OUT NUMBER) IS

BEGIN
  
  SELECT COUNT(*)
  INTO pExisteix
  FROM dispositiu d
  WHERE d.marca=pMarca AND d.model=pModel;
                
END existeixDispositiu;
/


/*
 * Mtode que verifica l'existncia d'un pas en la taula Pais.
 * {pPais} varchar amb el nom del pas.
 * {pExisteix} number que indica si existeix o no el registre a la taula.
 * 1 --> existeix, 0 --> NO existeix.
 */
CREATE OR REPLACE PROCEDURE existeixPais(pPais IN VARCHAR2, 
                                         pExisteix OUT NUMBER) IS

BEGIN
  
  SELECT COUNT(*)
  INTO pExisteix
  FROM pais p
  WHERE p.pais=pPais;
                
END existeixPais;
/


/*
 * Mtode que obte l'identificador d'un pas segons el seu nom.
 * {pNom} varchar amb el nom del pas.
 * {pId} varchar amb l'identificador del pas.
 */
CREATE OR REPLACE PROCEDURE obtenirIdPais(pNom IN VARCHAR2, pId OUT VARCHAR2) IS

BEGIN
  
  SELECT p.identificador
  INTO pId
  FROM pais p
  WHERE p.pais=pNom;
                
END obtenirIdPais;
/


/*
 * Mtode que verifica l'existncia d'un registre en la taula Dispositiu_Usuari.
 * {pImei} number amb l'imei (identificador) del dispositiu del usuari.
 * {pExisteix} number que indica si existeix o no el registre a la taula.
 * 1 --> existeix, 0 --> NO existeix.
 */
CREATE OR REPLACE PROCEDURE existeixDispositiuUsuari(pImei IN NUMBER, 
                            pExisteix OUT NUMBER) IS

BEGIN
  
  SELECT COUNT(*)
  INTO pExisteix
  FROM dispositiu_usuari du
  WHERE du.imei=pImei;
                
END existeixDispositiuUsuari;
/


/*
 * Mtode que indica si una persona es troba activada o desactivada en el 
 * sistema.
 * {pDni} varchar amb l'identificador de la persona.
 * {pActivada} number que indica si est activada o desactivada.
 * 1 --> activada, 0 --> desactivada.
 */
CREATE OR REPLACE PROCEDURE personaActivada(pDni IN VARCHAR2, 
                                            pActivada OUT NUMBER) IS

vBoolea VARCHAR2(5);

BEGIN
  
  select activo
  INTO vBoolea
  from persona p
  where p.identificador = pDni;
  
  IF vBoolea='true' THEN
    pActivada:=1;
  ELSE
    pActivada:=0;
  END IF;
                
END personaActivada;
/


/*
 * Mtode que activa una Persona en el sistema.
 * {pDni} varchar amb l'identificador de la persona.
 */
CREATE OR REPLACE PROCEDURE activarPersona(pDni IN VARCHAR2) IS

vActivado VARCHAR2(5):='true';

BEGIN
  
  UPDATE persona p SET p.activo=vActivado where p.identificador=pDni;  
                
END activarPersona;
/


/*
 * Mtode que desactiva una Persona en el sistema.
 * {pDni} varchar amb l'identificador de la persona.
 */
CREATE OR REPLACE PROCEDURE desactivarPersona(pDni IN VARCHAR2) IS

vActivado VARCHAR2(5):='false';

BEGIN
  
  UPDATE persona p SET p.activo=vActivado where p.identificador=pDni;  
                
END desactivarPersona;
/


/*
 * Mtode que crea un nou registre en la taula Persona.
 * {pDni} varchar amb l'identificador de la persona.
 * {pNom} varchar amb el nom de la persona.
 * {pAdreca} varchar amb l'adrea de la persona.
 * {pTelefon} number amb el nmero de telfon de la persona.
 * {pMail} varchar amb l'adrea de mail de la persona.
 * {pPais} varchar amb l'identificador del pas de la persona.
 */
CREATE OR REPLACE PROCEDURE crearPersona(pDni IN VARCHAR2, pNom IN VARCHAR2,
                            pAdreca IN VARCHAR2, pTelefon IN NUMBER,
                            pMail IN VARCHAR2, pPais IN VARCHAR2) IS

BEGIN
  
  INSERT INTO persona(identificador, nom, adreca, telefon, mail, pais)
  VALUES (pDni, pNom, pAdreca, pTelefon, pMail, pPais);
                
END crearPersona;
/


/*
 * Mtode que crea un nou registre en la taula Usuari.
 * {pDni} varchar amb l'identificador de l'usuari.
 * {pOperador} varchar amb el nom de l'operador telefnic de l'usuari.
 */
CREATE OR REPLACE PROCEDURE crearUsuari(pDni IN VARCHAR2, 
                                        pOperador IN VARCHAR) IS

BEGIN
  
  INSERT INTO usuari(identificador, operador) VALUES (pDni, pOperador);
                
END crearUsuari;
/


/*
 * Mtode que crea un nou registre en la taula Desenvolupador.
 * {pDni} varchar amb l'identificador del Desenvolupador.
 * {pRepresentant} varchar amb el nom del representant de l'empresa 
 * desenvolupadora.
 */
CREATE OR REPLACE PROCEDURE crearDesenvolupador(pDni IN VARCHAR2, 
                            pRepresentant IN VARCHAR) IS

BEGIN
  
  INSERT INTO desenvolupador(identificador, representant) 
  VALUES (pDni, pRepresentant);
                
END crearDesenvolupador;
/


/*
 * Mtode que modifica un registre en la taula Persona.
 * {pDni} varchar amb l'identificador de la persona a modificar.
 * {pNom} varchar amb el nom de la persona (el mateix o el nou).
 * {pAdreca} varchar amb la adrea de la persona (la mateixa o la nova).
 * {pTelefon} number amb el telfon de la persona (el mateix o el nou).
 * {pMail} varchar amb el mail de la persona (el mateix o el nou).
 * {pPais} varchar amb l'identificador del pas la persona (el mateix o el nou).
 */
CREATE OR REPLACE PROCEDURE cambiarPersona(pDni IN VARCHAR2, pNom IN VARCHAR2,
                            pAdreca IN VARCHAR2, pTelefon IN NUMBER,
                            pMail IN VARCHAR2, pPais IN VARCHAR2) IS

BEGIN
  
  UPDATE persona p SET p.nom=pNom, p.adreca=pAdreca, p.telefon=pTelefon, 
  p.mail=pMail, p.pais=pPais where p.identificador=pDni;
                
END cambiarPersona;
/


/*
 * Mtode que modifica un registre en la taula Usuari.
 * {pDni} varchar amb l'identificador de l'usuari a modificar.
 * {pOperador} varchar amb el nom de l'operador telefnic de l'usuari.
 */
CREATE OR REPLACE PROCEDURE cambiarUsuari(pDni IN VARCHAR2, 
                                          pOperador IN VARCHAR) IS

BEGIN
  
  UPDATE usuari u SET u.operador=pOperador where u.identificador=pDni;
                
END cambiarUsuari;
/


/*
 * Mtode que modifica un registre en la taula Desenvolupador.
 * {pDni} varchar amb l'identificador del desenvolupador a modificar.
 * {pRepresentant} varchar amb el nom del representant de l'empresa 
 * desenvolupadora.
 */
CREATE OR REPLACE PROCEDURE cambiarDesenvolupador(pDni IN VARCHAR2, 
                                                  pRepresentant IN VARCHAR) IS

BEGIN
  
  UPDATE desenvolupador d 
  SET d.representant=pRepresentant 
  WHERE d.identificador=pDni;
                
END cambiarDesenvolupador;
/


/*
 * Mtode que crea un nou registre en la taula Dispositiu_Usuari.
 * {pImei} number amb l'identificador del dispositiu del usuari.
 * {pDni} varchar amb l'idebtificador de l'usuari.
 * {pMarca} varchar amb la marca del dispositiu.
 * {pModel} varchar amb el model del dispositiu.
 */
CREATE OR REPLACE PROCEDURE crearDispositiuUsuari(pImei IN NUMBER, 
                            pDni IN VARCHAR2, pMarca IN VARCHAR2, 
                            pModel IN VARCHAR2) IS
                            
BEGIN
  
  INSERT INTO dispositiu_usuari(imei, usuari, marca, model) 
  VALUES (pImei, pDni, pMarca, pModel);
                
END crearDispositiuUsuari;
/


/*
 * Mtode que realitza l'alta d'un nou Usuari en el sistema.
 * {pDni} varchar amb l'identificador del nou usuari.
 * {pNom} varchar amb el nom de l'usuari.
 * {pAdreca} varchar amb la adrea de l'usuari.
 * {pTelefon} number amb el telfon de l'usuari.
 * {pMail} varchar amb el mail de l'usuari.
 * {pPais} varchar amb el nom del pas de l'usuari.
 * {pOperador} varchar amb el nom de l'operador telefnic de l'usuari.
 * {RSP} varchar amb un missatge que indica si l'execuci ha estat correcta o no
 * i la ra per la que no ha estat aix.
 */
CREATE OR REPLACE PROCEDURE altaUsuari(pDni IN VARCHAR2, pNom IN VARCHAR2,
                            pAdreca IN VARCHAR2, pTelefon IN NUMBER,
                            pMail IN VARCHAR2, pPais IN VARCHAR2,
                            pOperador IN VARCHAR2, RSP OUT VARCHAR2) IS

vExisteix NUMBER(1);
vActivada NUMBER(1);
vExisteixPais NUMBER(1);
vIdPais VARCHAR(2);
usuari_ya_existeix EXCEPTION;
pais_no_existeix EXCEPTION;
parametro_null EXCEPTION;

BEGIN
  
  /*
   * Es comproba que no existeix cap parametre null, en cas afirmatiu, es llana
   * una excepci.
   */  
  IF (pDni IS NULL OR pNom IS NULL OR pAdreca IS NULL OR pTelefon IS NULL OR 
       pMail IS NULL OR pPais IS NULL OR pOperador IS NULL) THEN 
    RAISE parametro_null;    
  END IF; 
  
  /*
   * Es comproba que existeix un pas amb aquest nom, en cas contrari, es llana
   * una excepci.
   */
  existeixPais(pPais, vExisteixPais);
  IF vExisteixPais=0 THEN
    RAISE pais_no_existeix;
  END IF;
  
  --s'obte l'identificador del pas.
  obtenirIdPais(pPais, vIdPais);
  
  /*
   * Es comproba si existeix una persona amb aquest dni:
   * Si existeix --> es comproba si est activada, si ho est es llana una
   *                 excepci per informar de que no es pot crear un usuari que
   *                 ja existeix, es cas contrari s'activa la persona i es
   *                 modifiquen les dades per si no son las mateixes que quan es
   *                 va donar de baixa.
   * Si NO existeix --> es crea la Persona i l'usuari.
   */
  existeixPersona(pDni, vExisteix);
  
  IF vExisteix=1 THEN
    personaActivada(pDni, vActivada);
    IF vActivada=1 THEN
      RAISE usuari_ya_existeix;
    ELSE
      activarPersona(pDni);
      cambiarPersona(pDni, pNom, pAdreca, pTelefon, pMail, vIdPais);
      cambiarUsuari(pDni, pOperador);
    END IF;
  ELSE
    crearPersona(pDni, pNom, pAdreca, pTelefon, pMail, vIdPais);
    crearUsuari(pDni, pOperador);
  END IF;
  
  -- S'afegeix al LOG la crida al mtode amb el seu resultat.
  RSP:='OK';
  afegirLog('Alta usuari', 'Dni: ' || pDni ||' Nom: ' || pNom || ' Adrea: '
            || pAdreca || ' Telefon: ' || pTelefon || ' Mail: ' || pMail ||
            ' Pais: ' || pPais || ' Operador: ' || pOperador, RSP);
  
  EXCEPTION
  
    /*
     * Desprs de cada excepci s'afegeix al LOG la crida al mtode amb l'error
     * que ha estat provocat.
     */
    WHEN usuari_ya_existeix THEN 
    RSP:='ERROR: USER ALREADY EXISTS';
    afegirLog('Alta usuari', 'Dni: ' || pDni ||' Nom: ' || pNom || ' Adrea: '
            || pAdreca || ' Telefon: ' || pTelefon || ' Mail: ' || pMail ||
            ' Pais: ' || pPais || ' Operador: ' || pOperador, RSP);
    
    WHEN pais_no_existeix THEN 
    RSP:='ERROR: USER COUNTRY NOT EXISTS';
    afegirLog('Alta usuari', 'Dni: ' || pDni ||' Nom: ' || pNom || ' Adrea: '
            || pAdreca || ' Telefon: ' || pTelefon || ' Mail: ' || pMail ||
            ' Pais: ' || pPais || ' Operador: ' || pOperador, RSP);
            
    WHEN parametro_null THEN 
    RSP:='ERROR: NULL PARAMETER';
    afegirLog('Alta usuari', 'Dni: ' || pDni ||' Nom: ' || pNom || ' Adrea: '
            || pAdreca || ' Telefon: ' || pTelefon || ' Mail: ' || pMail ||
            ' Pais: ' || pPais || ' Operador: ' || pOperador, RSP);
            
    WHEN OTHERS THEN
      IF (SQLCODE = -00001) THEN 
        RSP:='ERROR: DUPLICATED UNIQUE KEY';
        afegirLog('Alta usuari', 'Dni: ' || pDni ||' Nom: ' || pNom || 
            ' Adrea: ' || pAdreca || ' Telefon: ' || pTelefon || ' Mail: ' || 
            pMail || ' Pais: ' || pPais || ' Operador: ' || pOperador, RSP);
      END IF;
         
END altaUsuari;
/


/*
 * Mtode que realitza l'alta d'un nou Desenvolupador en el sistema.
 * {pDni} varchar amb l'identificador del nou desenvolupador.
 * {pNom} varchar amb el nom del desenvolupador.
 * {pAdreca} varchar amb la adrea del desenvolupador.
 * {pTelefon} number amb el telfon del desenvolupador.
 * {pMail} varchar amb el mail del desenvolupador.
 * {pPais} varchar amb el nom del pas del desenvolupador.
 * {pRepresentant} varchar amb el nom del representant de l'empresa 
 * desenvolupadora.
 * {RSP} varchar amb un missatge que indica si l'execuci ha estat correcta o no
 * i la ra per la que no ha estat aix.
 */
CREATE OR REPLACE PROCEDURE altaDesenvolupador(pDni IN VARCHAR2, 
                            pNom IN VARCHAR2, pAdreca IN VARCHAR2, 
                            pTelefon IN NUMBER, pMail IN VARCHAR2, 
                            pPais IN VARCHAR2, pRepresentant IN VARCHAR2, 
                            RSP OUT VARCHAR2) IS

vExisteix NUMBER(1);
vActivada NUMBER(1);
vExisteixPais NUMBER(1);
vIdPais VARCHAR(2);
desenvolupador_ya_existeix EXCEPTION;
pais_no_existeix EXCEPTION;
parametro_null EXCEPTION;

BEGIN
  
  /*
   * Es comproba que no existeix cap parametre null, en cas afirmatiu, es llana
   * una excepci.
   */  
  IF (pDni IS NULL OR pNom IS NULL OR pAdreca IS NULL OR pTelefon IS NULL OR 
      pMail IS NULL OR pPais IS NULL OR pRepresentant IS NULL) THEN 
    RAISE parametro_null;    
  END IF; 
  
  /*
   * Es comproba que existeix un pas amb aquest nom, en cas contrari, es llana
   * una excepci.
   */
  existeixPais(pPais, vExisteixPais);
  IF vExisteixPais=0 THEN
    RAISE pais_no_existeix;
  END IF;
  
  --s'obte l'identificador del pas.
  obtenirIdPais(pPais, vIdPais);
  
  /*
   * Es comproba si existeix una persona amb aquest dni:
   * Si existeix --> es comproba si est activada, si ho est es llana una
   *                 excepci per informar de que no es pot crear un 
   *                 desenvolupador que ja existeix, es cas contrari s'activa la 
   *                 persona i es modifiquen les dades per si no son las 
   *                 mateixes que quan es va donar de baixa.
   * Si NO existeix --> es crea la Persona i el Desenvolupador.
   */
  existeixPersona(pDni, vExisteix);
  
  IF vExisteix=1 THEN
    personaActivada(pDni, vActivada);
    IF vActivada=1 THEN
      RAISE desenvolupador_ya_existeix;
    ELSE
      activarPersona(pDni);
      cambiarPersona(pDni, pNom, pAdreca, pTelefon, pMail, vIdPais);
      cambiarDesenvolupador(pDni, pRepresentant);
    END IF;
  ELSE
    crearPersona(pDni, pNom, pAdreca, pTelefon, pMail, vIdPais);
    crearDesenvolupador(pDni, pRepresentant);
  END IF;
  
  -- S'afegeix al LOG la crida al mtode amb el seu resultat.
  RSP:='OK';
  afegirLog('Alta desenvolupador', 'Dni: ' || pDni ||' Nom: ' || pNom || 
            ' Adrea: ' || pAdreca || ' Telefon: ' || pTelefon || ' Mail: ' || 
            pMail || ' Pais: ' || pPais || ' Representant: ' || pRepresentant, 
            RSP);
  
  EXCEPTION
    
    /*
     * Desprs de cada excepci s'afegeix al LOG la crida al mtode amb l'error
     * que ha estat provocat.
     */
    WHEN desenvolupador_ya_existeix THEN RSP:='ERROR: DEVELOPER ALREADY EXISTS';
    afegirLog('Alta desenvolupador', 'Dni: ' || pDni ||' Nom: ' || pNom || 
            ' Adrea: ' || pAdreca || ' Telefon: ' || pTelefon || ' Mail: ' || 
            pMail || ' Pais: ' || pPais || ' Representant: ' || pRepresentant, 
            RSP);
    
    WHEN pais_no_existeix THEN RSP:='ERROR: DEVELOPER COUNTRY NOT EXISTS';
    afegirLog('Alta desenvolupador', 'Dni: ' || pDni ||' Nom: ' || pNom || 
            ' Adrea: ' || pAdreca || ' Telefon: ' || pTelefon || ' Mail: ' || 
            pMail || ' Pais: ' || pPais || ' Representant: ' || pRepresentant, 
            RSP);
            
    WHEN parametro_null THEN RSP:='ERROR: NULL PARAMETER';
    afegirLog('Alta desenvolupador', 'Dni: ' || pDni ||' Nom: ' || pNom || 
            ' Adrea: ' || pAdreca || ' Telefon: ' || pTelefon || ' Mail: ' || 
            pMail || ' Pais: ' || pPais || ' Representant: ' || pRepresentant, 
            RSP);
            
    WHEN OTHERS THEN
      IF (SQLCODE = -00001) THEN RSP:='ERROR: DUPLICATED UNIQUE KEY';
        afegirLog('Alta desenvolupador', 'Dni: ' || pDni ||' Nom: ' || pNom || 
            ' Adrea: ' || pAdreca || ' Telefon: ' || pTelefon || ' Mail: ' || 
            pMail || ' Pais: ' || pPais || ' Representant: ' || pRepresentant, 
            RSP);
      END IF;
                    
END altaDesenvolupador;
/


/*
 * Mtode que realitza la baixa d'un Usuari en el sistema.
 * {pDni} varchar amb l'identificador de l'usuari.
 * {RSP} varchar amb un missatge que indica si l'execuci ha estat correcta o no
 * i la ra per la que no ha estat aix.
 */
CREATE OR REPLACE PROCEDURE baixaUsuari(pDni IN VARCHAR2, RSP OUT VARCHAR2) IS

vExisteix NUMBER(1);
vActivada NUMBER(1);
parametro_null EXCEPTION;
usuari_no_existeix EXCEPTION;
usuari_desactivat EXCEPTION;

BEGIN

  /*
   * Es comproba que el parmetre pDni no s NULL, en cas de que ho sigui es
   * llana una excepci per indicar-lo.
   */  
  IF pDni IS NULL THEN 
    RAISE parametro_null;    
  END IF; 
  
  /*
   * Es comproba si existeix una persona amb aquest dni:
   * Si existeix --> es comproba si est activada, si NO ho est es llana una
   *                 excepci per informar de que no es pot donar de baixa un
   *                 usuari que ja est de baixa, es cas contrari es desactiva 
   *                 la  persona.
   * Si NO existeix --> es llana una excepci per informar que no existeix
   *                 l'usari que es vol donar de baixa.
   */
  esUsuari(pDni, vExisteix);
  
  IF vExisteix=1 THEN
    personaActivada(pDni, vActivada);
    IF vActivada=1 THEN
      desactivarPersona(pDni);
    ELSE
      RAISE usuari_desactivat;
    END IF;
  ELSE
    RAISE usuari_no_existeix;
  END IF;
  
  -- S'afegeix al LOG la crida al mtode amb el seu resultat.
  RSP:='OK';
  afegirLog('Baixa usuari', 'Dni: ' || pDni , RSP);
  
  EXCEPTION
    
    /*
     * Desprs de cada excepci s'afegeix al LOG la crida al mtode amb l'error
     * que ha estat provocat.
     */
    WHEN parametro_null THEN RSP:='ERROR: NULL PARAMETER';
    afegirLog('Baixa usuari', 'Dni: ' || pDni , RSP);
    
    WHEN usuari_desactivat THEN RSP:='ERROR: USER ALREADY DISABLED';
    afegirLog('Baixa usuari', 'Dni: ' || pDni , RSP);

    WHEN usuari_no_existeix THEN RSP:='ERROR: USER NOT EXISTS';
    afegirLog('Baixa usuari', 'Dni: ' || pDni , RSP);
                    
END baixaUsuari;
/


/*
 * Mtode que realitza la baixa d'un Desenvolupador en el sistema.
 * {pDni} varchar amb l'identificador del desenvolupador.
 * {RSP} varchar amb un missatge que indica si l'execuci ha estat correcta o no
 * i la ra per la que no ha estat aix.
 */
CREATE OR REPLACE PROCEDURE baixaDesenvolupador(pDni IN VARCHAR2, 
                                                RSP OUT VARCHAR2) IS

vExisteix NUMBER(1);
vActivada NUMBER(1);
parametro_null EXCEPTION;
desenvolupador_no_existeix EXCEPTION;
desenvolupador_desactivat EXCEPTION;

BEGIN
  
  /*
   * Es comproba que el parmetre pDni no s NULL, en cas de que ho sigui es
   * llana una excepci per indicar-lo.
   */  
  IF pDni IS NULL THEN 
    RAISE parametro_null;    
  END IF; 
  
  /*
   * Es comproba si existeix una persona amb aquest dni:
   * Si existeix --> es comproba si est activada, si NO ho est es llana una
   *                 excepci per informar de que no es pot donar de baixa un
   *                 desenvolupador que ja est de baixa, es cas contrari es 
   *                 desactiva la persona.
   * Si NO existeix --> es llana una excepci per informar que no existeix el
   *                 desenvolupador que es vol donar de baixa.
   */
  esDesenvolupador(pDni, vExisteix);
  
  IF vExisteix=1 THEN
    personaActivada(pDni, vActivada);
    IF vActivada=1 THEN
      desactivarPersona(pDni);
    ELSE
      RAISE desenvolupador_desactivat;
    END IF;
  ELSE
    RAISE desenvolupador_no_existeix;
  END IF;
  
  -- S'afegeix al LOG la crida al mtode amb el seu resultat.
  RSP:='OK';
  afegirLog('Baixa desenvolupador', 'Dni: ' || pDni , RSP);
  
  EXCEPTION
    
    /*
     * Desprs de cada excepci s'afegeix al LOG la crida al mtode amb l'error
     * que ha estat provocat.
     */
    WHEN parametro_null THEN RSP:='ERROR: NULL PARAMETER';
    afegirLog('Baixa usuari', 'Dni: ' || pDni , RSP);
    
    WHEN desenvolupador_desactivat THEN RSP:='ERROR: DEVELOPER ALREADY DISABLED';
    afegirLog('Baixa desenvolupador', 'Dni: ' || pDni , RSP);

    WHEN desenvolupador_no_existeix THEN RSP:='ERROR: DEVELOPER NOT EXISTS';
    afegirLog('Baixa desenvolupador', 'Dni: ' || pDni , RSP);
                    
END baixaDesenvolupador;
/


/*
 * Mtode que realitza modificacions en un Usuari del sistema.
 * {pDni} varchar amb l'identificador de l'usuari.
 * {pNom} varchar amb el nom de l'usuari.
 * {pAdreca} varchar amb la adrea de l'usuari.
 * {pTelefon} number amb el telfon de l'usuari.
 * {pMail} varchar amb el mail de l'usuari.
 * {pPais} varchar amb el nom del pas de l'usuari.
 * {pOperador} varchar amb el nom de l'operador telefnic de l'usuari.
 * {RSP} varchar amb un missatge que indica si l'execuci ha estat correcta o no
 * i la ra per la que no ha estat aix.
 */
CREATE OR REPLACE PROCEDURE modificarUsuari(pDni IN VARCHAR2, pNom IN VARCHAR2,
                            pAdreca IN VARCHAR2, pTelefon IN NUMBER,
                            pMail IN VARCHAR2, pPais IN VARCHAR2,
                            pOperador IN VARCHAR2, RSP OUT VARCHAR2) IS

vExisteix NUMBER(1);
vActivada NUMBER(1);
vExisteixPais NUMBER(1);
vIdPais VARCHAR2(2);
pais_no_existeix EXCEPTION;
usuari_no_existeix EXCEPTION;
usuari_desactivat EXCEPTION;
parametro_null EXCEPTION;

BEGIN
    
  /*
   * Es comproba que no existeix cap parametre null, en cas afirmatiu, es llana
   * una excepci.
   */  
  IF (pDni IS NULL OR pNom IS NULL OR pAdreca IS NULL OR pTelefon IS NULL OR 
      pMail IS NULL OR pPais IS NULL OR pOperador IS NULL) THEN 
    RAISE parametro_null;    
  END IF; 
  
  /*
   * Es comproba que existeix un pas amb aquest nom, en cas contrari, es llana
   * una excepci.
   */
  existeixPais(pPais, vExisteixPais);
  IF vExisteixPais=0 THEN
    RAISE pais_no_existeix;
  END IF;
  
  --s'obte l'identificador del pas
  obtenirIdPais(pPais, vIdPais);
  
  /*
   * Es comproba si existeix una persona que sigui usuari amb aquest dni:
   * Si existeix --> es comproba si est activada, si NO ho est es llana una
   *                 excepci per informar de que no es pot modificar un usuari
   *                 de baixa, es cas contrari es modifiquen les dades de la 
   *                 persona i de l'usuari.
   * Si NO existeix --> es llana una excepci per indicar que no es pot 
   *                    modificar un usuari que no existeix.
   */ 

  esUsuari(pDni, vExisteix);
  
  IF vExisteix=1 THEN
    personaActivada(pDni, vActivada);
    IF vActivada=1 THEN
      cambiarPersona(pDni, pNom, pAdreca, pTelefon, pMail, vIdPais);
      cambiarUsuari(pDni, pOperador);
    ELSE
      RAISE usuari_desactivat;
    END IF;
  ELSE
    RAISE usuari_no_existeix;
  END IF;
  
  -- S'afegeix al LOG la crida al mtode amb el seu resultat.
  RSP:='OK';
  afegirLog('Modificar Usuari','Dni: ' || pDni ||' Nom: ' || pNom || ' Adrea: '
            || pAdreca || ' Telefon: ' || pTelefon || ' Mail: ' || pMail ||
            ' Pais: ' || pPais || ' Operador: ' || pOperador, RSP);
  
  EXCEPTION
    
    /*
     * Desprs de cada excepci s'afegeix al LOG la crida al mtode amb l'error
     * que ha estat provocat.
     */
    WHEN usuari_desactivat THEN RSP:='ERROR: USER DISABLED';
    afegirLog('Modificar Usuari', 'Dni: ' || pDni ||' Nom: ' || pNom || 
            ' Adrea: ' || pAdreca || ' Telefon: ' || pTelefon || ' Mail: ' || 
            pMail || ' Pais: ' || pPais || ' Operador: ' || pOperador, RSP);

    WHEN usuari_no_existeix THEN RSP:='ERROR: USER NOT EXISTS';
    afegirLog('Modificar Usuari', 'Dni: ' || pDni ||' Nom: ' || pNom || 
            ' Adrea: ' || pAdreca || ' Telefon: ' || pTelefon || ' Mail: ' || 
            pMail || ' Pais: ' || pPais || ' Operador: ' || pOperador, RSP);
    
    WHEN pais_no_existeix THEN RSP:='ERROR: USER COUNTRY NOT EXISTS';
    afegirLog('Modificar Usuari', 'Dni: ' || pDni ||' Nom: ' || pNom || 
            ' Adrea: ' || pAdreca || ' Telefon: ' || pTelefon || ' Mail: ' || 
            pMail || ' Pais: ' || pPais || ' Operador: ' || pOperador, RSP);
    
    WHEN parametro_null THEN RSP:='ERROR: NULL PARAMETER';
    afegirLog('Modificar Usuari', 'Dni: ' || pDni ||' Nom: ' || pNom || 
            ' Adrea: ' || pAdreca || ' Telefon: ' || pTelefon || ' Mail: ' || 
            pMail || ' Pais: ' || pPais || ' Operador: ' || pOperador, RSP);
            
    WHEN OTHERS THEN
      IF (SQLCODE = -00001) THEN RSP:= 'ERROR: DUPLICATED UNIQUE KEY';
        afegirLog('Modificar Usuari', 'Dni: ' || pDni ||' Nom: ' || pNom || 
            ' Adrea: ' || pAdreca || ' Telefon: ' || pTelefon || ' Mail: ' || 
            pMail || ' Pais: ' || pPais || ' Operador: ' || pOperador, RSP);
      END IF;
                    
END modificarUsuari;
/


/*
 * Mtode que realitza modificacions en un Desenvolupador del sistema.
 * {pDni} varchar amb l'identificador del desenvolupador.
 * {pNom} varchar amb el nom del desenvolupador.
 * {pAdreca} varchar amb la adrea del desenvolupador.
 * {pTelefon} number amb el telfon del desenvolupador.
 * {pMail} varchar amb el mail del desenvolupador.
 * {pPais} varchar amb el nom del pas del desenvolupador.
 * {pRepresentant} varchar amb el nom del representant de l'empresa 
 * desenvolupadora.
 * {RSP} varchar amb un missatge que indica si l'execuci ha estat correcta o no
 * i la ra per la que no ha estat aix.
 */
CREATE OR REPLACE PROCEDURE modificarDesenvolupador(pDni IN VARCHAR2, 
                            pNom IN VARCHAR2, pAdreca IN VARCHAR2, 
                            pTelefon IN NUMBER, pMail IN VARCHAR2, 
                            pPais IN VARCHAR2, pRepresentant IN VARCHAR2, 
                            RSP OUT VARCHAR2) IS

vExisteix NUMBER(1);
vActivada NUMBER(1);
vExisteixPais NUMBER(1);
vIdPais VARCHAR2(2);
pais_no_existeix EXCEPTION;
desenvolupador_no_existeix EXCEPTION;
desenvolupador_desactivat EXCEPTION;
parametro_null EXCEPTION;

BEGIN
    
  /*
   * Es comproba que no existeix cap parametre null, en cas afirmatiu, es llana
   * una excepci.
   */  
  IF (pDni IS NULL OR pNom IS NULL OR pAdreca IS NULL OR pTelefon IS NULL OR 
      pMail IS NULL OR pPais IS NULL OR pRepresentant IS NULL) THEN 
    RAISE parametro_null;    
  END IF; 
  
  /*
   * Es comproba que existeix un pas amb aquest nom, en cas contrari, es llana
   * una excepci.
   */
  existeixPais(pPais, vExisteixPais);
  IF vExisteixPais=0 THEN
    RAISE pais_no_existeix;
  END IF;
  
  --s'obte l'identificador del pas.
  obtenirIdPais(pPais, vIdPais);
  
  /*
   * Es comproba si existeix una persona que sigui desenvolupador amb aquest 
   * dni:
   * Si existeix --> es comproba si est activada, si NO ho est es llana una
   *                 excepci per informar de que no es pot modificar un
   *                 desenvolupador de baixa, es cas contrari es modifiquen les 
   *                 dades de la persona i del desenvolupador.
   * Si NO existeix --> es llana una excepci per indicar que no es pot 
   *                    modificar un usuari que no existeix.
   */ 

  esDesenvolupador(pDni, vExisteix);
  
  IF vExisteix=1 THEN
    personaActivada(pDni, vActivada);
    IF vActivada=1 THEN
      cambiarPersona(pDni, pNom, pAdreca, pTelefon, pMail, vIdPais);
      cambiarDesenvolupador(pDni, pRepresentant);
    ELSE
      RAISE desenvolupador_desactivat;
    END IF;
  ELSE
    RAISE desenvolupador_no_existeix;
  END IF;
  
  -- S'afegeix al LOG la crida al mtode amb el seu resultat
  RSP:='OK';
  afegirLog('Modificar Desenvolupador', 'Dni: ' || pDni ||' Nom: ' || pNom || 
            ' Adrea: ' || pAdreca || ' Telefon: ' || pTelefon || ' Mail: ' || 
            pMail || ' Pais: ' || pPais || ' Representant: ' || pRepresentant, 
            RSP);
  
  EXCEPTION
  
    /*
     * Desprs de cada excepci s'afegeix al LOG la crida al mtode amb l'error
     * que ha estat provocat
     */
    WHEN desenvolupador_desactivat THEN RSP:='ERROR: DEVELOPER DISABLED';
    afegirLog('Modificar Desenvolupador', 'Dni: ' || pDni ||' Nom: ' || pNom || 
            ' Adrea: ' || pAdreca || ' Telefon: ' || pTelefon || ' Mail: ' || 
            pMail || ' Pais: ' || pPais || ' Representant: ' || pRepresentant, 
            RSP);

    WHEN desenvolupador_no_existeix THEN RSP:='ERROR: DEVELOPER NOT EXISTS';
    afegirLog('Modificar Desenvolupador', 'Dni: ' || pDni ||' Nom: ' || pNom || 
            ' Adrea: ' || pAdreca || ' Telefon: ' || pTelefon || ' Mail: ' || 
            pMail || ' Pais: ' || pPais || ' Representant: ' || pRepresentant, 
            RSP);
    
    WHEN pais_no_existeix THEN RSP:='ERROR: DEVELOPER COUNTRY NOT EXISTS';
    afegirLog('Modificar Desenvolupador', 'Dni: ' || pDni ||' Nom: ' || pNom || 
            ' Adrea: ' || pAdreca || ' Telefon: ' || pTelefon || ' Mail: ' || 
            pMail || ' Pais: ' || pPais || ' Representant: ' || pRepresentant, 
            RSP);
            
    WHEN parametro_null THEN RSP:='ERROR: NULL PARAMETER';
    afegirLog('Modificar Desenvolupador', 'Dni: ' || pDni ||' Nom: ' || pNom || 
            ' Adrea: ' || pAdreca || ' Telefon: ' || pTelefon || ' Mail: ' || 
            pMail || ' Pais: ' || pPais || ' Representant: ' || pRepresentant, 
            RSP);
            
    WHEN OTHERS THEN
      IF (SQLCODE = -00001) THEN RSP:= 'ERROR: DUPLICATED UNIQUE KEY';
        afegirLog('Modificar Desenvolupador', 'Dni: ' || pDni ||' Nom: ' || 
            pNom || ' Adrea: ' || pAdreca || ' Telefon: ' || pTelefon || 
            ' Mail: ' || pMail || ' Pais: ' || pPais || ' Representant: ' || 
            pRepresentant, RSP);
      END IF;
                    
END modificarDesenvolupador;
/


/*
 * Mtode que asigna un dispositiu nic i especific a un Usuari del sistema.
 * {pDni} varchar amb l'identificador de l'usuari.
 * {pMarca} varchar amb la marca del dispositiu.
 * {pModel} varchar amb el model del dispositiu.
 * {pImei} number amb l'identificador del dispositiu de l'usuari.
 * {RSP} varchar amb un missatge que indica si l'execuci ha estat correcta o no
 * i la ra per la que no ha estat aix.
 */
CREATE OR REPLACE PROCEDURE assignarDispositiuUsuari(pDni IN VARCHAR2, 
                            pMarca IN VARCHAR2, pModel IN VARCHAR2, 
                            pImei IN NUMBER, RSP OUT VARCHAR2) IS

vEsUsuari NUMBER(1);
vExisteixDispositiu NUMBER(1);
vExisteixDispositiuUsuari NUMBER(1);
usuari_no_existeix EXCEPTION;
dispositiu_no_existeix EXCEPTION;
dispositiu_ja_asignat EXCEPTION;
parametro_null EXCEPTION;

BEGIN
  
  /*
   * Es comproba que no existeix cap parametre null, en cas afirmatiu, es llana
   * una excepci.
   */
  IF (pDni IS NULL OR pMarca IS NULL OR pModel IS NULL OR pImei IS NULL) THEN 
    RAISE parametro_null;    
  END IF; 
 
  /*
   * Es comproba si existeix una persona que sigui usuari amb aquest dni:
   * Si existeix --> es comproba que existeix un dispositiu amb aquesta marca i 
   *                 model
   *             Si existeix --> es comproba si existeix algn dispositiu amb 
   *                             aquest IMEI, en cas afirmatiu es llana una 
   *                             excepci indicant aquest dispositiu ja ha estat
   *                             asignat, en cas contrari, se asigna el 
   *                             dispositiu a l'usuari
   *             Si NO existeix --> es llana una excepci indicant que no es t
   *                                constncia d'un dispositiu amb aquesta marca
   *                                i model
   * Si NO existeix --> es llana una excepci per indicar que no es pot asignar
   *                 un dispositiu a un usuari que no existeix.
   */ 
  esUsuari(pDni, vEsUsuari);
  
  IF vEsUsuari=1 THEN
    existeixdispositiu(pMarca, pModel, vExisteixDispositiu);
    IF vExisteixDispositiu=1 THEN
      existeixDispositiuUsuari(pImei, vExisteixDispositiuUsuari);
      IF vExisteixDispositiuUsuari=1 THEN
        RAISE dispositiu_ja_asignat;
      ELSE
        crearDispositiuUsuari(pImei, pDni, pMarca, pModel);
      END IF;
    ELSE
      RAISE dispositiu_no_existeix;
    END IF;
  ELSE
    RAISE usuari_no_existeix;
  END IF;
  
  -- S'afegeix al LOG la crida al mtode amb el seu resultat.
  RSP:='OK';
  afegirLog('Asignar Dispositivo a Usuario', 'Imei: ' || pImei || ' Usuari: ' 
            || pDni || ' Marca: ' || pMarca || ' Model: ' || pModel, RSP);
  
  EXCEPTION
    
    /*
     * Desprs de cada excepci s'afegeix al LOG la crida al mtode amb l'error
     * que ha estat provocat.
     */
    WHEN usuari_no_existeix THEN RSP:='ERROR: USER NOT EXISTS';
    afegirLog('Asignar Dispositivo a Usuario', 'Imei: ' || pImei || ' Usuari: ' 
            || pDni || ' Marca: ' || pMarca || ' Model: ' || pModel, RSP);
    
    WHEN dispositiu_no_existeix THEN RSP:='ERROR: DEVICE NOT EXISTS';
    afegirLog('Asignar Dispositivo a Usuario', 'Imei: ' || pImei || ' Usuari: ' 
            || pDni || ' Marca: ' || pMarca || ' Model: ' || pModel, RSP);
    
    WHEN dispositiu_ja_asignat THEN RSP:='ERROR: DEVICE ALREADY ASSIGNED';
    afegirLog('Asignar Dispositivo a Usuario', 'Imei: ' || pImei || ' Usuari: ' 
            || pDni || ' Marca: ' || pMarca || ' Model: ' || pModel, RSP);
            
    WHEN parametro_null THEN RSP:='ERROR: NULL PARAMETER';
    afegirLog('Asignar Dispositivo a Usuario', 'Imei: ' || pImei || ' Usuari: ' 
            || pDni || ' Marca: ' || pMarca || ' Model: ' || pModel, RSP);
                    
END assignarDispositiuUsuari;
/


/*
 * Mtode que asigna un dispositiu que pertany a un Usuari a altre Usuari del
 * sistema.
 * {pNouPropietari} varchar amb l'identificador del nou usuari propietari del 
 * dispositiu.
 * {pImei} number amb l'identificador del dispositiu de l'usuari.
 * {RSP} varchar amb un missatge que indica si l'execuci ha estat correcta o no
 * i la ra per la que no ha estat aix.
 */
CREATE OR REPLACE PROCEDURE modificarPropietariDispositiu(pNouPropietari IN 
                            VARCHAR2, pImei IN NUMBER, RSP OUT VARCHAR2) IS

vEsUsuari NUMBER(1);
vExisteixDispositiuUsuari NUMBER(1);
usuari_no_existeix EXCEPTION;
dispositiu_no_existeix EXCEPTION;
parametro_null EXCEPTION;

BEGIN
  
  /*
   * Es comproba que no existeix cap parametre null, en cas afirmatiu, es llana
   * una excepci.
   */
  IF (pNouPropietari IS NULL OR pImei IS NULL) THEN 
    RAISE parametro_null;    
  END IF; 
 
  /*
   * Es comproba si existeix una persona que sigui usuari amb aquest dni:
   *     Si existeix --> es comproba si existeix algn dispositiu amb 
   *                     aquest IMEI, en cas afirmatiu es modifica el usuari
   *                     propietari del mateix, en cas contrari es llana una
   *                     excepci per indicar que no existeix cap dispositiu
   *                     amb aquest imei.
   *     Si NO existeix --> es llana una excepci per indicar que no es pot 
   *                        asignar un dispositiu a un usuari que no existeix.
   */ 
  esUsuari(pNouPropietari, vEsUsuari);
  
  IF vEsUsuari=1 THEN
    existeixDispositiuUsuari(pImei, vExisteixDispositiuUsuari);
    IF vExisteixDispositiuUsuari=1 THEN
      UPDATE dispositiu_usuari SET usuari=pNouPropietari WHERE imei=pImei;
    ELSE
      RAISE dispositiu_no_existeix;
    END IF;
  ELSE
    RAISE usuari_no_existeix;
  END IF;
  
  -- S'afegeix al LOG la crida al mtode amb el seu resultat.
  RSP:='OK';
  afegirLog('Modificar Propietari de un dispositiu', 'Imei: ' || pImei || 
            ' Nou usuari: ' || pNouPropietari, RSP);
  
  EXCEPTION
    
    /*
     * Desprs de cada excepci s'afegeix al LOG la crida al mtode amb l'error
     * que ha estat provocat.
     */
    WHEN usuari_no_existeix THEN RSP:='ERROR: USER NOT EXISTS';
    afegirLog('Modificar Propietari de un dispositiu', 'Imei: ' || pImei || 
              ' Nou usuari: ' || pNouPropietari, RSP);
    
    WHEN dispositiu_no_existeix THEN RSP:='ERROR: DEVICE NOT EXISTS';
    afegirLog('Modificar Propietari de un dispositiu', 'Imei: ' || pImei || 
              ' Nou usuari: ' || pNouPropietari, RSP);
            
    WHEN parametro_null THEN RSP:='ERROR: NULL PARAMETER';
    afegirLog('Modificar Propietari de un dispositiu', 'Imei: ' || pImei || 
              ' Nou usuari: ' || pNouPropietari, RSP);
                    
END modificarPropietariDispositiu;
/


/*
 * Mtode que torna el pais i el usuari al que pertany un dispositiu d'usuari.
 * {pDispositiu} number amb l'identificador del dispositiu (imei).
 * {pPais} varchar amb el nom del pais on est registrat el propietari del 
 * dispositiu.
 * {pUsuari} varchar amb l'identificador de l'usuari propietari del dispositiu.
 */
CREATE OR REPLACE PROCEDURE obtenirDadesDispositiu(pDispositiu IN NUMBER, 
                            pPais OUT VARCHAR2, pUsuari OUT VARCHAR2) IS

BEGIN
  
  SELECT pa.pais, p.identificador
  INTO pPais, pUsuari
  FROM dispositiu_usuari du INNER JOIN persona p
  ON du.usuari = p.identificador AND du.imei = pDispositiu
  INNER JOIN pais pa
  ON pa.identificador = p.pais;
                
END obtenirDadesDispositiu;
/


/*
 * Mtode que verifica l'existncia d'un registre en la taula Aplicacio.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pExisteix} number que indica si existeix o no el registre a la taula.
 * 1 --> existeix, 0 --> NO existeix.
 */
CREATE OR REPLACE PROCEDURE existeixAplicacio(pAplicacio IN VARCHAR2, 
                            pExisteix OUT NUMBER) IS

BEGIN
  
  SELECT COUNT(*)
  INTO pExisteix
  FROM aplicacio app
  WHERE app.aplicacio = pAplicacio;
                
END existeixAplicacio;
/


/*
 * Mtode que verifica l'existncia d'un registre en la taula Versio.
 * {pVersio} varchar amb la versi.
 * {pExisteix} number que indica si existeix o no el registre a la taula.
 * 1 --> existeix, 0 --> NO existeix.
 */
CREATE OR REPLACE PROCEDURE existeixVersio(pVersio IN VARCHAR2, 
                            pExisteix OUT NUMBER) IS

BEGIN
  
  SELECT COUNT(*)
  INTO pExisteix
  FROM versio v
  WHERE v.versio = pVersio;
                
END existeixVersio;
/


/*
 * Mtode que verifica l'existncia d'un registre en la taula 
 * Aplicacio_Versionada.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pExisteix} number que indica si existeix o no el registre a la taula.
 * 1 --> existeix, 0 --> NO existeix.
 */
CREATE OR REPLACE PROCEDURE existeixAplicacioVersionada(pAplicacio IN VARCHAR2, 
                            pVersio IN VARCHAR2, pExisteix OUT NUMBER) IS

BEGIN 
    
  SELECT COUNT(*)
  INTO pExisteix
  FROM aplicacio_versionada appv
  WHERE appv.aplicacio = pAplicacio AND appv.versio=pVersio;
  
END existeixAplicacioVersionada;
/


/*
 * Mtode que indica si una aplicacio versionada es troba activada o desactivada 
 * en el sistema.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pActivada} number que indica si est activada o desactivada.
 * 1 --> activada, 0 --> desactivada.
 */
CREATE OR REPLACE PROCEDURE aplicacioVersionadaActivada(pAplicacio IN VARCHAR2, 
                            pVersio IN VARCHAR2, pActivada OUT NUMBER) IS

vBoolea VARCHAR2(5);

BEGIN 
    
  SELECT appv.activa
  INTO vBoolea
  FROM aplicacio_versionada appv
  WHERE appv.aplicacio = pAplicacio AND appv.versio = pVersio;
  
  IF vBoolea='true' THEN
    pActivada:=1;
  ELSE
    pActivada:=0;
  END IF;
  
END aplicacioVersionadaActivada;
/


/*
 * Mtode que activa una Aplicaci Versionada en el sistema.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 */
CREATE OR REPLACE PROCEDURE activarAplicacioVersionada(pAplicacio IN VARCHAR2, 
                            pVersio IN VARCHAR2) IS

vActivado VARCHAR2(5):='true';

BEGIN
  
  UPDATE aplicacio_versionada appv 
  SET appv.activa=vActivado 
  where appv.aplicacio = pAplicacio AND appv.versio = pVersio;  
  
END activarAplicacioVersionada;
/


/*
 * Mtode que desactiva una Aplicaci Versionada en el sistema.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 */
CREATE OR REPLACE PROCEDURE desactivarAplicacioVersionada
                                (pAplicacio IN VARCHAR2, pVersio IN VARCHAR2) IS

vActivado VARCHAR2(5):='false';

BEGIN
  
  UPDATE aplicacio_versionada appv 
  SET appv.activa=vActivado 
  where appv.aplicacio = pAplicacio AND appv.versio = pVersio;  
  
END desactivarAplicacioVersionada;
/


/*
 * Mtode que crea un nou registre en la taula Aplicacio.
 * {pNom} varchar amb el nom de l'aplicaci.
 */
CREATE OR REPLACE PROCEDURE crearAplicacio(pNom IN VARCHAR2) IS

BEGIN
  
  INSERT INTO aplicacio(aplicacio) VALUES (pNom);
                
END crearAplicacio;
/


/*
 * Mtode que crea un nou registre en la taula Versio.
 * {pVersio} varchar amb la versi.
 */
CREATE OR REPLACE PROCEDURE crearVersio(pVersio IN VARCHAR2) IS

BEGIN
  
  INSERT INTO versio(versio) VALUES (pVersio);
                
END crearVersio;
/


/*
 * Mtode que crea un nou registre en la taula Aplicacio Versionada.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pResoluci} number que identifica la resoluci mnima de l'aplicaci.
 * {pUrl} varchar que cont l'adrea de l'URL on es troba el video de 
 * demostraci.
 */
CREATE OR REPLACE PROCEDURE crearAplicacioVersionada(pAplicacio IN VARCHAR2, 
                            pVersio IN VARCHAR2, pResolucio IN NUMBER, 
                            pUrl IN VARCHAR2) IS

vAvui DATE;

BEGIN
  
  --s'obt la data actual per emmagatzemar-la com data de pujada.
  SELECT TO_DATE(TO_CHAR(SYSDATE, 'DD-MM-YYYY'), 'DD-MM-YYYY') 
  INTO vAvui FROM DUAL;  
  
  INSERT INTO aplicacio_versionada(aplicacio, versio, resolucio_minima, url, 
  data_pujada) VALUES (pAplicacio, pVersio, pResolucio, pUrl, vAvui);
                
END crearAplicacioVersionada;
/


/*
 * Mtode que modifica un registre en la taula Aplicaci Versionada.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pResoluci} number que identifica la resoluci mnima de l'aplicaci.
 * {pUrl} varchar que cont l'adrea de l'URL on es troba el video de 
 * demostraci.
 */
CREATE OR REPLACE PROCEDURE cambiarAplicacioVersionada(pAplicacio IN VARCHAR2, 
                            pVersio IN VARCHAR, pResolucio IN NUMBER, 
                            pUrl IN VARCHAR2) IS

BEGIN
  
  --noms es permet variar la resoluci i la url d'una aplicaci versionada.
  UPDATE aplicacio_versionada appv 
  SET appv.resolucio_minima=pResolucio, appv.url=pUrl 
  WHERE appv.aplicacio=pAplicacio AND appv.versio=pVersio;
                
END cambiarAplicacioVersionada;
/


/*
 * Mtode que realitza l'alta d'una nova Aplicaci Versionada en el sistema
 * {pNom} varchar amb el nom de l'aplicaci
 * {pVersio} varchar amb la versi de l'aplicaci
 * {pResoluci} number que referencia a la resoluci mnima de l'aplicaci
 * {pUrl} vachar amb l'url on es troba el vdeo demostratiu de l'aplicaci 
 * pot ser NULL.
 * {RSP} varchar amb un missatge que indica si l'execuci ha estat correcta o no
 * i la ra per la que no ha estat aix.
 */
CREATE OR REPLACE PROCEDURE altaAplicacioVersionada(pNom IN VARCHAR2,
                            pVersio IN VARCHAR2, pResolucio IN NUMBER, 
                            pUrl IN VARCHAR2, RSP OUT VARCHAR2) IS

vExisteixAppVer NUMBER(1);
vExisteixAplicacio NUMBER(1);
vExisteixVersio NUMBER(1);
vActivada NUMBER(1);
aplicacio_versionada_existeix EXCEPTION;
parametro_null EXCEPTION;

BEGIN
  
  /*
   * Es comproba que no existeix cap parametre null excepte l'URL, en cas 
   * afirmatiu, es llana una excepci.
   */ 
  IF (pNom IS NULL OR pVersio IS NULL OR pResolucio IS NULL) THEN 
    RAISE parametro_null;
  END IF;
  
  /*
   * Es comproba si existeix una aplicaci amb aquest nom.
   * Es comproba si existeix aquesta versi.
   * Es comproba si ja existeix una apliacin versionada igual.
   * Es comproba si l'aplicaci versionada es troba activa.
   * 
   * Existeix aplicaci, versio i activada --> llana excepci aplicaci ja 
   *                                           existeix.
   * Existeix aplicaci, versio i NO activada --> activa i modifica l'aplicaci 
   *                                              versionada.
   * Existeix aplicaci y NO existeix versi --> crea Versi i aplicaci 
   *                                             vesionada.
   * NO existeix aplicaci i SI existeix versi --> crea Aplicaci i aplicaci 
   *                                                verisonada.
   * NO existeix aplicaci i NO existeix versi --> crea Aplicaci, Versi i 
   *                                                aplicaci versionada.
   */ 
  
  existeixAplicacioVersionada(pNom, pVersio, vExisteixAppVer); 
  existeixAplicacio(pNom, vExisteixAplicacio);
  existeixVersio(pVersio, vExisteixVersio);
  
  IF vExisteixAppVer = 1 THEN
    aplicacioVersionadaActivada(pNom, pVersio, vActivada);
    IF vActivada = 1 THEN
      RAISE aplicacio_versionada_existeix;
    ELSE
      activarAplicacioVersionada(pNom, pVersio);
      cambiarAplicacioVersionada(pNom, pVersio, pResolucio, pUrl);
    END IF;
  ELSE  
    IF vExisteixAplicacio = 1 THEN
      IF vExisteixVersio = 0 THEN
        crearVersio(pVersio);
      END IF;
      crearAplicacioVersionada(pNom, pVersio, pResolucio, pUrl);
    ELSE
      IF vExisteixVersio = 0 THEN
        crearVersio(pVersio);
      END IF;
      crearAplicacio(pNom);
      crearAplicacioVersionada(pNom, pVersio, pResolucio, pUrl);
    END IF;
  END IF;
  
  -- S'afegeix al LOG la crida al mtode amb el seu resultat.
  RSP:='OK';
  afegirLog('Alta aplicacio versionada', 'Nom: ' || pNom ||' Versio: ' || 
            pVersio || ' Resolucio mnima: ' || pResolucio || ' URL: ' || pUrl, 
            RSP);
  
  EXCEPTION
    
    /*
     * Desprs de cada excepci s'afegeix al LOG la crida al mtode amb l'error
     * que ha estat provocat
     */
    WHEN aplicacio_versionada_existeix THEN 
    RSP:='ERROR: APPLICATION ALREADY EXISTS';
    afegirLog('Alta aplicacio versionada', 'Nom: ' || pNom ||' Versio: ' || 
            pVersio || ' Resolucio mnima: ' || pResolucio || ' URL: ' || pUrl, 
            RSP);
            
    WHEN parametro_null THEN RSP:='ERROR: NULL PARAMETER';
    afegirLog('Alta aplicacio versionada', 'Nom: ' || pNom ||' Versio: ' || 
            pVersio || ' Resolucio mnima: ' || pResolucio || ' URL: ' || pUrl, 
            RSP);
         
END altaAplicacioVersionada;
/


/*
 * Mtode que realitza la baixa d'una Aplicaci Versionada en el sistema.
 * {pNom} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi de l'aplicaci.
 * {RSP} varchar amb un missatge que indica si l'execuci ha estat correcta o no
 * i la ra per la que no ha estat aix.
 */
CREATE OR REPLACE PROCEDURE baixaAplicacioVersionada(pNom IN VARCHAR2,
                            pVersio IN VARCHAR2, RSP OUT VARCHAR2) IS

vExisteixAplicacioVersionada NUMBER(1);
vActivada NUMBER(1);
aplicacio_no_existeix EXCEPTION;
aplicacio_desactivada EXCEPTION;
parametro_null EXCEPTION;

BEGIN
  
  /*
   * Es comproba que no existeix cap parametre null, en cas afirmatiu, es llana 
   * una excepci.
   */   
  IF (pNom IS NULL OR pVersio IS NULL) THEN 
    RAISE parametro_null;
  END IF;
  
  /*
   * Es comproba si existeix una aplicaci versionada amb aquest nom i versi
   * Es comproba si l'aplicaci versionada es troba activa.
   * 
   * Existeix aplicaci i activa --> es desactiva l'aplicaci versionada.
   * Existeix aplicaci i NO activa --> llana excepci aplicaci ja de baixa.
   * NO existeix aplicaci --> llana excepci aplicaci no existeix.
   */ 
  existeixAplicacioVersionada(pNom, pVersio, vExisteixAplicacioVersionada);  
  
  IF vExisteixAplicacioVersionada = 1 THEN
    aplicacioVersionadaActivada(pNom, pVersio, vActivada);
    IF vActivada = 1 THEN
      desactivarAplicacioVersionada(pNom, pVersio);
    ELSE
      RAISE aplicacio_desactivada;
    END IF;
  ELSE
    RAISE aplicacio_no_existeix;  
  END IF;
  
  -- S'afegeix al LOG la crida al mtode amb el seu resultat.
  RSP:='OK';
  afegirLog('Baixa aplicacio versionada', 'Nom: ' || pNom ||' Versio: ' || 
            pVersio, RSP);
  
  EXCEPTION
    
    /*
     * Desprs de cada excepci s'afegeix al LOG la crida al mtode amb l'error
     * que ha estat provocat.
     */
    WHEN aplicacio_no_existeix THEN RSP:='ERROR: APPLICATION NOT EXISTS';
    afegirLog('Baixa aplicacio versionada', 'Nom: ' || pNom ||' Versio: ' || 
            pVersio, RSP);
    
    WHEN aplicacio_desactivada THEN RSP:='ERROR: APPLICATION ALREADY DISABLED';
    afegirLog('Baixa aplicacio versionada', 'Nom: ' || pNom ||' Versio: ' || 
            pVersio, RSP);
            
    WHEN parametro_null THEN RSP:='ERROR: NULL PARAMETER';
    afegirLog('Baixa aplicacio versionada', 'Nom: ' || pNom ||' Versio: ' || 
            pVersio, RSP);
         
END baixaAplicacioVersionada;
/


/*
 * Mtode que realitza la modificaci d'una Aplicaci Versionada en el sistema.
 * {pNom} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi de l'aplicaci.
 * {pResoluci} number que identifica la resoluci mnima de l'aplicaci.
 * {pUrl} varchar que cont l'adrea de l'URL on es troba el video de 
 * demostraci.
 * {RSP} varchar amb un missatge que indica si l'execuci ha estat correcta o no
 * i la ra per la que no ha estat aix.
 */
CREATE OR REPLACE PROCEDURE modificarAplicacioVersionada(pNom IN VARCHAR2,
                            pVersio IN VARCHAR2, pResolucio IN NUMBER,
                            pUrl IN VARCHAR2, RSP OUT VARCHAR2) IS

vExisteixAplicacioVersionada NUMBER(1);
vActivada NUMBER(1);
aplicacio_no_existeix EXCEPTION;
aplicacio_desactivada EXCEPTION;
parametro_null EXCEPTION;

BEGIN
  
  /*
   * Es comproba que no existeix cap parametre null excepte l'url, en cas 
   * afirmatiu, es llana una excepci.
   */   
  IF (pNom IS NULL OR pVersio IS NULL OR pResolucio IS NULL) THEN 
    RAISE parametro_null;
  END IF;
  
  /*
   * Es comproba si existeix una aplicaci versionada amb aquest nom i versi
   * Es comproba si l'aplicaci versionada es troba activa.
   * 
   * Existeix aplicaci i activa --> modifica l'aplicaci versionada.
   * Existeix aplicaci i NO activa --> llana excepci aplicaci desactivada
   *                                    indicant que no es pot modificar.
   * NO existeix aplicaci --> llana excepci aplicaci no existeix.
   */ 
  existeixAplicacioVersionada(pNom, pVersio, vExisteixAplicacioVersionada);
  
  IF vExisteixAplicacioVersionada = 1 THEN
    aplicacioVersionadaActivada(pNom, pVersio, vActivada);
    IF vActivada = 1 THEN
      cambiarAplicacioVersionada(pNom, pVersio, pResolucio, pUrl);
    ELSE
      RAISE aplicacio_desactivada;
    END IF;
  ELSE
    RAISE aplicacio_no_existeix;  
  END IF;
  
  -- S'afegeix al LOG la crida al mtode amb el seu resultat.
  RSP:='OK';
  afegirLog('Modificar aplicacio versionada', 'Nom: ' || pNom ||' Versio: ' || 
            pVersio || ' Resolucio Mnima: ' || pResolucio || ' URL: ' || pUrl, 
            RSP);
  
  EXCEPTION
    
    /*
     * Desprs de cada excepci s'afegeix al LOG la crida al mtode amb l'error
     * que ha estat provocat.
     */
    WHEN aplicacio_no_existeix THEN RSP:='ERROR: APPLICATION NOT EXISTS';
    afegirLog('Modificar aplicacio versionada', 'Nom: ' || pNom ||' Versio: ' || 
            pVersio || ' Resolucio Mnima: ' || pResolucio || ' URL: ' || pUrl, 
            RSP);
    
    WHEN aplicacio_desactivada THEN RSP:='ERROR: APPLICATION IS DISABLED';
    afegirLog('Modificar aplicacio versionada', 'Nom: ' || pNom ||' Versio: ' || 
            pVersio || ' Resolucio Mnima: ' || pResolucio || ' URL: ' || pUrl, 
            RSP);
            
    WHEN parametro_null THEN RSP:='ERROR: NULL PARAMETER';
    afegirLog('Modificar aplicacio versionada', 'Nom: ' || pNom ||' Versio: ' || 
            pVersio || ' Resolucio Mnima: ' || pResolucio || ' URL: ' || pUrl, 
            RSP);
         
END modificarAplicacioVersionada;
/


/*
 * Mtode que verifica l'existncia d'un registre en la taula Preu.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pPais} varchar amb l'identificador del pas.
 * {pExisteix} number que indica si existeix o no el registre a la taula.
 * 1 --> existeix, 0 --> NO existeix.
 */
CREATE OR REPLACE PROCEDURE existeixPreu(pAplicacio IN VARCHAR2, 
                            pVersio IN VARCHAR2,pPais IN VARCHAR2, 
                            pExisteix OUT NUMBER) IS

BEGIN
  
  SELECT COUNT(*)
  INTO pExisteix
  FROM preu p
  WHERE p.aplicacio=pAplicacio AND p.versio=pVersio AND p.Pais=pPais;
                
END existeixPreu;
/


/*
 * Mtode que crea un nou registre en la taula Preu.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pPais} varchar amb l'identificador del pas.
 * {pPreu} number amb el preu de l'aplicaci en el pas donat.
 */
CREATE OR REPLACE PROCEDURE crearPreu(pAplicacio IN VARCHAR2, 
                            pVersio IN VARCHAR2, pPais IN VARCHAR2, 
                            pPreu IN NUMBER) IS

BEGIN
  
  INSERT INTO preu(aplicacio, versio, pais, preu)
  VALUES (pAplicacio, pVersio, pPais, pPreu);
                
END crearPreu;
/


/*
 * Mtode que modifica un registre en la taula Preu.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pPais} varchar amb l'identificador del pas.
 * {pPreu} number amb el preu de l'aplicaci en el pas donat (el mateix o el 
 * nou).
 */
CREATE OR REPLACE PROCEDURE cambiarPreu(pAplicacio IN VARCHAR2, 
                            pVersio IN VARCHAR2, pPais IN VARCHAR2, 
                            pPreu IN NUMBER) IS

BEGIN
  
  UPDATE preu p 
  SET p.preu=pPreu 
  WHERE p.aplicacio=pAplicacio AND p.versio=pVersio AND p.pais=pPais;
                
END cambiarPreu;
/


/*
 * Mtode que asigna un preu a una aplicaci segons el pas.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pPais} varchar amb el nom del pas.
 * {pPreu} number amb el preu de l'aplicaci en el pas donat.
 * {RSP} varchar amb un missatge que indica si l'execuci ha estat correcta o no
 * i la ra per la que no ha estat aix.
 */
CREATE OR REPLACE PROCEDURE assignarPreuAplicacio(pAplicacio IN VARCHAR2, 
                            pVersio IN VARCHAR2, pPais IN VARCHAR2, 
                            pPreu IN NUMBER, RSP OUT VARCHAR2) IS

vExisteixAplicacio NUMBER(1);
vExisteixPais NUMBER(1);
vExisteixPreu NUMBER(1);
vIdPais VARCHAR2(2);
aplicacio_no_existeix EXCEPTION;
pais_no_existeix EXCEPTION;
parametro_null EXCEPTION;
preu_negatiu EXCEPTION;

BEGIN
  
  /*
   * Es comproba que no existeix cap parametre null, en cas afirmatiu, es llana
   * una excepci.
   */
  IF (pAplicacio IS NULL OR pVersio IS NULL OR pPais IS NULL 
                                                      OR pPreu IS NULL) THEN 
    RAISE parametro_null;    
  END IF;
  
  /*
   * Es comproba que el preu que es vol asignar s positiu, en cas contrari es 
   * llana una excepci.
   */
  IF (pPreu<0) THEN
    RAISE preu_negatiu;
  END IF;
  
  /*
   * Es comproba que existeix un pas amb aquest nom, en cas contrari, es llana
   * una excepci.
   */
  existeixPais(pPais, vExisteixPais);
  IF vExisteixPais=0 THEN
    RAISE pais_no_existeix;
  END IF;
  
  --s'obte l'identificador del pas
  obtenirIdPais(pPais, vIdPais);
 
  /*
   * Es comproba si existeix la aplicaci versionada:
   * Si existeix --> es comproba si ja existeix un preu per aquesta aplicaci i 
   *                 pas
   *             Si existeix --> es modifica el preu de l'aplicaci.
   *             Si NO existeix --> es crea un nou preu per aquesta aplicaci en
   *                                 aquest pas.
   * Si NO existeix --> es llana una excepci per indicar que no es pot asignar
   *                 un preu a una aplicaci que no existeix.
   */ 
  existeixAplicacioVersionada(pAplicacio, pVersio, vExisteixAplicacio);
  
  IF vExisteixAplicacio=1 THEN
    existeixPreu(pAplicacio, pVersio, vIdPais, vExisteixPreu);
    IF vExisteixPreu=1 THEN
      cambiarPreu(pAplicacio, pVersio, vIdPais, pPreu);      
    ELSE
      crearPreu(pAplicacio, pVersio, vIdPais, pPreu);
    END IF;
  ELSE
    RAISE aplicacio_no_existeix;
  END IF;
  
  -- S'afegeix al LOG la crida al mtode amb el seu resultat.
  RSP:='OK';
  afegirLog('Asignar Preu a Aplicaci', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' Pas: ' || pPais || ' Preu: ' || pPreu, 
            RSP);
  
  EXCEPTION
    
    /*
     * Desprs de cada excepci s'afegeix al LOG la crida al mtode amb l'error
     * que ha estat provocat.
     */    
    WHEN aplicacio_no_existeix THEN RSP:='ERROR: APPLICATION NOT EXISTS';
    afegirLog('Asignar Preu a Aplicaci', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' Pas: ' || pPais || ' Preu: ' || pPreu, 
            RSP);
    
    WHEN pais_no_existeix THEN RSP:='ERROR: COUNTRY NOT EXISTS';
    afegirLog('Asignar Preu a Aplicaci', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' Pas: ' || pPais || ' Preu: ' || pPreu, 
            RSP);
            
    WHEN parametro_null THEN RSP:='ERROR: NULL PARAMETER';
    afegirLog('Asignar Preu a Aplicaci', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' Pas: ' || pPais || ' Preu: ' || pPreu, 
            RSP);
    
    WHEN preu_negatiu THEN RSP:='ERROR: NEGATIVE PRICE VALUE';
    afegirLog('Asignar Preu a Aplicaci', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' Pas: ' || pPais || ' Preu: ' || pPreu, 
            RSP);
                    
END assignarPreuAplicacio;
/


/*
 * Mtode que verifica l'existncia d'un registre en la taula 
 * Desenvolupador_Aplicaci
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pDni} varchar amb l'identificador del desenvolupador.
 * {pExisteix} number que indica si existeix o no el registre a la taula.
 * 1 --> existeix, 0 --> NO existeix.
 */
CREATE OR REPLACE PROCEDURE existeixDesenvolupadorApp(pAplicacio IN VARCHAR2, 
                            pVersio IN VARCHAR2, pDni IN VARCHAR2, 
                            pExisteix OUT NUMBER) IS

BEGIN
  
  SELECT COUNT(*)
  INTO pExisteix
  FROM desenvolupador_aplicacio da
  WHERE da.aplicacio=pAplicacio AND da.versio=pVersio 
                                              AND da.desenvolupador=pDni;
                
END existeixDesenvolupadorApp;
/


/*
 * Mtode que crea un nou registre en la taula Desenvolupador Aplicaci.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pDni} varchar amb l'identificador del desenvolupador.
 */
CREATE OR REPLACE PROCEDURE crearDesenvolupadorAplicacio(pAplicacio IN VARCHAR2, 
                            pVersio IN VARCHAR2, pDni IN VARCHAR2) IS

BEGIN
  
  INSERT INTO desenvolupador_aplicacio(aplicacio, versio, desenvolupador)
  VALUES (pAplicacio, pVersio, pDni);
                
END crearDesenvolupadorAPlicacio;
/


/*
 * Mtode que asigna un desenvolupador a una aplicaci.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pDni} varchar amb l'identificador del desenvolupador.
 * {RSP} varchar amb un missatge que indica si l'execuci ha estat correcta o no
 * i la ra per la que no ha estat aix.
 */
CREATE OR REPLACE PROCEDURE asignarDesenvolupadorAplicacio
                            (pAplicacio IN VARCHAR2, pVersio IN VARCHAR2, 
                            pDni IN VARCHAR2, RSP OUT VARCHAR2) IS

vExisteixAplicacio NUMBER(1);
vEsDesenvolupador NUMBER(1);
vExisteixDesenvolupadorApp NUMBER(1);
aplicacio_no_existeix EXCEPTION;
desenvolupador_no_existeix EXCEPTION;
desenvolupador_asignat EXCEPTION;
parametro_null EXCEPTION;

BEGIN
  
  /*
   * Es comproba que no existeix cap parametre null, en cas afirmatiu, es llana
   * una excepci.
   */
  IF (pAplicacio IS NULL OR pVersio IS NULL OR pDni IS NULL) THEN 
    RAISE parametro_null;    
  END IF;
  
  /*
   * Es comproba que el dni perteneix a un desenvolupador, si no, es llana una
   * excepci.
   */
  esDesenvolupador(pDni, vEsDesenvolupador);
  IF (vEsDesenvolupador=0) THEN 
    RAISE desenvolupador_no_existeix;    
  END IF;
 
  /*
   * Es comproba si existeix la aplicaci versionada:
   * Si existeix --> es comproba si ja existeix el desenvolupador en aquesta 
   *                 aplicaci
   *             Si existeix --> es llana una excepci per indicar que ja est
   *                             asignat
   *             Si NO existeix --> s'asigna el desenvolupador a l'aplicaci
   * Si NO existeix --> es llana una excepci per indicar que no es pot asignar
   *                 un desenvolupador a una aplicaci que no existeix.
   */ 
  existeixAplicacioVersionada(pAplicacio, pVersio, vExisteixAplicacio);
  
  IF vExisteixAplicacio=1 THEN
    existeixDesenvolupadorApp(pAplicacio, pVersio, pDni, 
                                                    vExisteixDesenvolupadorApp);
    IF vExisteixDesenvolupadorApp=1 THEN
      RAISE desenvolupador_asignat;
    ELSE
      crearDesenvolupadorAplicacio(pAplicacio, pVersio, pDni);
    END IF;
  ELSE
    RAISE aplicacio_no_existeix;
  END IF;
  
  -- S'afegeix al LOG la crida al mtode amb el seu resultat.
  RSP:='OK';
  afegirLog('Asignar Desenvolupador a Aplicaci', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' Desenvolupador: ' || pDni, RSP);
  
  EXCEPTION
    
    /*
     * Desprs de cada excepci s'afegeix al LOG la crida al mtode amb l'error
     * que ha estat provocat.
     */    
    WHEN aplicacio_no_existeix THEN RSP:='ERROR: APPLICATION NOT EXISTS';
    afegirLog('Asignar Desenvolupador a Aplicaci', 'Aplicaci: ' || pAplicacio 
              || ' Versio: ' || pVersio || ' Desenvolupador: ' || pDni, RSP);
    
    WHEN desenvolupador_no_existeix THEN RSP:='ERROR: DEVELOPER NOT EXISTS';
    afegirLog('Asignar Desenvolupador a Aplicaci', 'Aplicaci: ' || pAplicacio 
              || ' Versio: ' || pVersio || ' Desenvolupador: ' || pDni, RSP);
    
    WHEN desenvolupador_asignat THEN RSP:='ERROR: DEVELOPER ALREADY ASSIGNED';
    afegirLog('Asignar Desenvolupador a Aplicaci', 'Aplicaci: ' || pAplicacio 
              || ' Versio: ' || pVersio || ' Desenvolupador: ' || pDni, RSP);
            
    WHEN parametro_null THEN RSP:='ERROR: NULL PARAMETER';
    afegirLog('Asignar Desenvolupador a Aplicaci', 'Aplicaci: ' || pAplicacio 
              || ' Versio: ' || pVersio || ' Desenvolupador: ' || pDni, RSP);
                    
END asignarDesenvolupadorAplicacio;
/


/*
 * Mtode que verifica l'existncia d'un registre en la taula Idioma.
 * {pIdioma} varchar amb el nom del idioma.
 * {pExisteix} number que indica si existeix o no el registre a la taula.
 * 1 --> existeix, 0 --> NO existeix.
 */
CREATE OR REPLACE PROCEDURE existeixIdioma(pIdioma IN VARCHAR2, 
                                           pExisteix OUT NUMBER) IS

BEGIN
  
  SELECT COUNT(*)
  INTO pExisteix
  FROM idioma i
  WHERE i.idioma=pIdioma;
                
END existeixIdioma;
/


/*
 * Mtode que verifica l'existncia d'un registre en la taula Descripci.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pIdioma} varchar amb l'identificador del idioma.
 * {pExisteix} number que indica si existeix o no el registre a la taula.
 * 1 --> existeix, 0 --> NO existeix.
 */
CREATE OR REPLACE PROCEDURE existeixDescripcio(pAplicacio IN VARCHAR2, 
                            pVersio IN VARCHAR2, pIdioma IN VARCHAR2, 
                            pExisteix OUT NUMBER) IS

BEGIN
  
  SELECT COUNT(*)
  INTO pExisteix
  FROM descripcio d
  WHERE d.aplicacio= pAplicacio AND d.versio=pVersio AND d.idioma=pIdioma;
                
END existeixDescripcio;
/


/*
 * Mtode que obte l'identificador d'un idioma segons el seu nom.
 * {pIdioma} varchar amb el nom del idioma.
 * {pId} varchar amb l'identificador del idioma.
 */
CREATE OR REPLACE PROCEDURE obtenirIdIdioma(pIdioma IN VARCHAR2, 
                                            pId OUT VARCHAR2) IS

BEGIN
  
  SELECT i.identificador
  INTO pId
  FROM idioma i
  WHERE i.idioma=pIdioma;
                
END obtenirIdIdioma;
/


/*
 * Mtode que crea un nou registre en la taula Descripci.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pIdioma} varchar amb l'identificador del idioma.
 * {pDescripcio} varchar amb la descripci de l'aplicaci en el idioma donat.
 */
CREATE OR REPLACE PROCEDURE crearDescripcio(pAplicacio IN VARCHAR2, 
                            pVersio IN VARCHAR2, pIdioma IN VARCHAR2, 
                            pDescripcio IN VARCHAR2) IS

BEGIN
  
  INSERT INTO descripcio(aplicacio, versio, idioma, descripcio)
  VALUES (pAplicacio, pVersio, pIdioma, pDescripcio);
                
END crearDescripcio;
/


/*
 * Mtode que modifica un registre en la taula Descripci.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pIdioma} varchar amb l'identificador del idioma.
 * {pDescripcio} varchar amb la descripci de l'aplicaci en el idioma donat.
 */
CREATE OR REPLACE PROCEDURE cambiarDescripcio(pAplicacio IN VARCHAR2, 
                            pVersio IN VARCHAR2, pIdioma IN VARCHAR2, 
                            pDescripcio IN VARCHAR2) IS

BEGIN
  
  UPDATE descripcio d SET d.descripcio=pDescripcio where d.aplicacio=pAplicacio 
  AND d.versio=pVersio AND d.idioma=pIdioma;
                
END cambiarDescripcio;
/


/*
 * Mtode que asigna una descripci a una aplicaci.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pIdioma} varchar amb el nom del idioma.
 * {pDescripcio} varchar amb la descripci de l'aplicaci en el idioma donat.
 * {RSP} varchar amb un missatge que indica si l'execuci ha estat correcta o no
 * i la ra per la que no ha estat aix.
 */
CREATE OR REPLACE PROCEDURE assignarDescripcioAplicacio(pAplicacio IN VARCHAR2, 
                            pVersio IN VARCHAR2, pIdioma IN VARCHAR2, 
                            pDescripcio IN VARCHAR2, RSP OUT VARCHAR2) IS

vExisteixAplicacio NUMBER(1);
vExisteixIdioma NUMBER(1);
vExisteixDescripcio NUMBER(1);
vIdIdioma VARCHAR(2);
aplicacio_no_existeix EXCEPTION;
idioma_no_existeix EXCEPTION;
parametro_null EXCEPTION;

BEGIN
  
  /*
   * Es comproba que no existeix cap parametre null, en cas afirmatiu, es llana
   * una excepci.
   */
  IF (pAplicacio IS NULL OR pVersio IS NULL OR pIdioma IS NULL OR 
                                                      pDescripcio IS NULL) THEN 
    RAISE parametro_null;    
  END IF;
  
  /*
   * Es comproba que existeix un idioma amb aquest nom, en cas contrari, es 
   * llana una excepci.
   */
  existeixIdioma(pIdioma, vExisteixIdioma);
  IF vExisteixIdioma=0 THEN
    RAISE idioma_no_existeix;
  END IF;
  
  --s'obte l'identificador del idioma.
  obtenirIdIdioma(pIdioma, vIdIdioma);
 
  /*
   * Es comproba si existeix la aplicaci versionada:
   * Si existeix --> es comproba si ja existeix la descripci en aquest idioma.
   *             Si existeix --> es modifica la descripci.
   *             Si NO existeix --> s'asigna la nova descripci a l'aplicaci.
   * Si NO existeix --> es llana una excepci per indicar que no es pot asignar
   *                 una descripci a una aplicaci que no existeix.
   */ 
  existeixAplicacioVersionada(pAplicacio, pVersio, vExisteixAplicacio);
  
  IF vExisteixAplicacio=1 THEN
    existeixDescripcio(pAplicacio, pVersio, vIdIdioma, vExisteixDescripcio);
    IF vExisteixDescripcio=1 THEN
      cambiarDescripcio(pAplicacio, pVersio, vIdIdioma, pDescripcio);
    ELSE
      crearDescripcio(pAplicacio, pVersio, vIdIdioma, pDescripcio);
    END IF;
  ELSE
    RAISE aplicacio_no_existeix;
  END IF;
  
  -- S'afegeix al LOG la crida al mtode amb el seu resultat.
  RSP:='OK';
  afegirLog('Asignar Descripci a Aplicaci', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' Idioma: ' || pIdioma, RSP);
  
  EXCEPTION
    
    /*
     * Desprs de cada excepci s'afegeix al LOG la crida al mtode amb l'error
     * que ha estat provocat.
     */    
    WHEN aplicacio_no_existeix THEN RSP:='ERROR: APPLICATION NOT EXISTS';
    afegirLog('Asignar Descripci a Aplicaci', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' Idioma: ' || pIdioma, RSP);
    
    WHEN idioma_no_existeix THEN RSP:='ERROR: LANGUAGE NOT EXISTS';
    afegirLog('Asignar Descripci a Aplicaci', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' Idioma: ' || pIdioma, RSP);
            
    WHEN parametro_null THEN RSP:='ERROR: NULL PARAMETER';
    afegirLog('Asignar Descripci a Aplicaci', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' Idioma: ' || pIdioma, RSP);
                    
END assignarDescripcioAplicacio;
/


/*
 * Mtode que verifica l'existncia d'un registre en la taula SSOO.
 * {pSSOO} varchar amb el nom del sistema oparatiu.
 * {pExisteix} number que indica si existeix o no el registre a la taula.
 * 1 --> existeix, 0 --> NO existeix.
 */
CREATE OR REPLACE PROCEDURE existeixSSOO(pSSOO IN VARCHAR2, 
                                         pExisteix OUT NUMBER) IS

BEGIN
  
  SELECT COUNT(*)
  INTO pExisteix
  FROM ssoo so
  WHERE so.ssoo=pSSOO;
                
END existeixSSOO;
/


/*
 * Mtode que obte l'identificador d'un sistema operatiu segons el seu nom.
 * {pSSOO} varchar amb el nom del sistema operatiu.
 * {pId} number amb l'identificador del sistema operatiu.
 */
CREATE OR REPLACE PROCEDURE obtenirIdSSOO(pSSOO IN VARCHAR2, pId OUT NUMBER) IS

BEGIN
  
  SELECT so.identificador
  INTO pId
  FROM ssoo so
  WHERE so.ssoo=pSSOO;
                
END obtenirIdSSOO;
/


/*
 * Mtode que verifica l'existncia d'un registre en la taula ssoo_aplicaci.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pSSOO} varchar amb l'identificador del sistema operatiu.
 * {pExisteix} number que indica si existeix o no el registre a la taula.
 * 1 --> existeix, 0 --> NO existeix.
 */
CREATE OR REPLACE PROCEDURE existeixSSOOAplicacio(pAplicacio IN VARCHAR2, 
                            pVersio IN VARCHAR2, pSSOO IN NUMBER, 
                            pExisteix OUT NUMBER) IS

BEGIN
  
  SELECT COUNT(*)
  INTO pExisteix
  FROM ssoo_aplicacio soa
  WHERE soa.aplicacio= pAplicacio AND soa.versio=pVersio AND soa.ssoo=pSSOO;
                
END existeixSSOOAplicacio;
/


/*
 * Mtode que crea un nou registre en la taula SSOO_Aplicaci.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pSSOO} number amb l'identificador del sistema operatiu.
 * {pBinari} varchar amb la ruta on es troba el arxiu binari de l'aplicaci per
 *           aquest sistema operatiu.
 * {pMida} number amb la mida del arxiu de l'aplicaci per aquest sistema 
 * operatiu.
 */
CREATE OR REPLACE PROCEDURE crearSSOOAplicacio(pAplicacio IN VARCHAR2, 
                            pVersio IN VARCHAR2, pSSOO IN NUMBER, 
                            pBinari IN VARCHAR2, pMida IN NUMBER) IS

BEGIN
  
  INSERT INTO ssoo_aplicacio(aplicacio, versio, ssoo, binari, mida)
  VALUES (pAplicacio, pVersio, pSSOO, pbinari, pMida);
                
END crearSSOOAplicacio;
/


/*
 * Mtode que modifica un registre en la taula SSOO_Aplicaci.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pSSOO} number amb l'identificador del sistema operatiu.
 * {pBinari} varchar amb la ruta on es troba el arxiu binari de l'aplicaci per
 *           aquest sistema operatiu.
 * {pMida} number amb la mida del arxiu de l'aplicaci per aquest sistema 
 *         operatiu.
 */
CREATE OR REPLACE PROCEDURE cambiarSSOOAplicacio(pAplicacio IN VARCHAR2, 
                            pVersio IN VARCHAR2, pSSOO IN NUMBER, 
                            pBinari IN VARCHAR2, pMida IN NUMBER) IS

BEGIN
  
  UPDATE ssoo_aplicacio soa SET soa.binari=pBinari, soa.mida=pMida where 
  soa.aplicacio=pAplicacio AND soa.versio=pVersio AND soa.ssoo=pSSOO;
                
END cambiarSSOOAplicacio;
/


/*
 * Mtode que asigna un sistema operatiu a una aplicaci.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pSSOO} varchar amb el nom del sistema operatiu.
 * {pBinari} varchar amb la ruta on es troba el arxiu binari de l'aplicaci per
 *           aquest sistema operatiu.
 * {pMida} number amb la mida del arxiu de l'aplicaci per aquest sistema 
 * operatiu.
 * {RSP} varchar amb un missatge que indica si l'execuci ha estat correcta o no
 * i la ra per la que no ha estat aix.
 */
CREATE OR REPLACE PROCEDURE assignarSSOOAplicacio(pAplicacio IN VARCHAR2, 
                            pVersio IN VARCHAR2, pSSOO IN VARCHAR2, 
                            pBinari IN VARCHAR2, pMida IN NUMBER, 
                            RSP OUT VARCHAR2) IS

vExisteixAplicacio NUMBER(1);
vExisteixSSOO NUMBER(1);
vExisteixSSOOAplicacio NUMBER(1);
vIdSSOO NUMBER(5);
aplicacio_no_existeix EXCEPTION;
ssoo_no_existeix EXCEPTION;
parametro_null EXCEPTION;
mida_negativa EXCEPTION;

BEGIN
  
  /*
   * Es comproba que no existeix cap parametre null, en cas afirmatiu, es llana
   * una excepci.
   */
  IF (pAplicacio IS NULL OR pVersio IS NULL OR pSSOO IS NULL OR pBinari IS NULL
      OR pMida IS NULL) THEN 
    RAISE parametro_null;    
  END IF;
  
  /*
   * Es comproba que la mida de l'arxiu binari s positiva, en cas contrari es 
   * llana una excepci.
   */
  IF (pMida<0) THEN
    RAISE mida_negativa;
  END IF;
  
  /*
   * Es comproba que existeix un sistema operatiu amb aquest nom, en cas 
   * contrari, es llana una excepci.
   */
  existeixSSOO(pSSOO, vExisteixSSOO);
  IF vExisteixSSOO=0 THEN
    RAISE ssoo_no_existeix;
  END IF;
  
  --s'obte l'identificador del sistema operatiu.
  obtenirIdSSOO(pSSOO, vIdSSOO);
 
  /*
   * Es comproba si existeix la aplicaci versionada:
   * Si existeix --> es comproba si ja existeix el sistema operatiu per aquesta 
   *                 aplicaci.
   *             Si existeix --> es modifica la el sistema operatiu de 
   *                             l'aplicaci.
   *             Si NO existeix --> s'asigna un nou sistema operatiu a 
   *                                l'aplicaci.
   * Si NO existeix --> es llana una excepci per indicar que no es pot asignar
   *                 un sistema operatiu a una aplicaci que no existeix.
   */ 
  existeixAplicacioVersionada(pAplicacio, pVersio, vExisteixAplicacio);
  
  IF vExisteixAplicacio=1 THEN
    existeixSSOOAplicacio(pAplicacio, pVersio, vIdSSOO, vExisteixSSOOAplicacio);
    IF vExisteixSSOOAplicacio=1 THEN
      cambiarSSOOAplicacio(pAplicacio, pVersio, vIdSSOO, pBinari, pMida);
    ELSE
      crearSSOOAplicacio(pAplicacio, pVersio, vIdSSOO, pBinari, pMida);
    END IF;
  ELSE
    RAISE aplicacio_no_existeix;
  END IF;
  
  -- S'afegeix al LOG la crida al mtode amb el seu resultat
  RSP:='OK';
  afegirLog('Asignar SSOO a Aplicaci', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' SSOO: ' || pSSOO || ' Binari: ' || 
            pBinari || ' Mida: ' || pMida, RSP);
  
  EXCEPTION
    
    /*
     * Desprs de cada excepci s'afegeix al LOG la crida al mtode amb l'error
     * que ha estat provocat
     */    
    WHEN aplicacio_no_existeix THEN RSP:='ERROR: APPLICATION NOT EXISTS';
    afegirLog('Asignar SSOO a Aplicaci', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' SSOO: ' || pSSOO || ' Binari: ' || 
            pBinari || ' Mida: ' || pMida, RSP);
    
    WHEN ssoo_no_existeix THEN RSP:='ERROR: SSOO NOT EXISTS';
    afegirLog('Asignar SSOO a Aplicaci', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' SSOO: ' || pSSOO || ' Binari: ' || 
            pBinari || ' Mida: ' || pMida, RSP);
            
    WHEN parametro_null THEN RSP:='ERROR: NULL PARAMETER';
    afegirLog('Asignar SSOO a Aplicaci', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' SSOO: ' || pSSOO || ' Binari: ' || 
            pBinari || ' Mida: ' || pMida, RSP);
    
    WHEN mida_negativa THEN RSP:='ERROR: NEGATIVE SIZE VALUE';
    afegirLog('Asignar SSOO a Aplicaci', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' SSOO: ' || pSSOO || ' Binari: ' || 
            pBinari || ' Mida: ' || pMida, RSP);
                    
END assignarSSOOAplicacio;
/


/*
 * Mtode que verifica l'existncia d'un registre en la taula pagament.
 * {pPagament} varchar amb el tipus de pagament.
 * {pExisteix} number que indica si existeix o no el registre a la taula.
 * 1 --> existeix, 0 --> NO existeix.
 */
CREATE OR REPLACE PROCEDURE existeixPagament(pPagament IN VARCHAR2, 
                                             pExisteix OUT NUMBER) IS

BEGIN
  
  SELECT COUNT(*)
  INTO pExisteix
  FROM pagament p
  WHERE p.pagament=pPagament;
                
END existeixPagament;
/


/*
 * Mtode que torna el preu de la aplicaci descarregada segons el pas al que 
 * pertany l'amo del dispositiu on s'ha realitzat.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pDispositiu} number amb l'identificador del dispositiu de usuari (imei).
 * {pPreuPagat} number amb el preu.
 */
CREATE OR REPLACE PROCEDURE obtenirPreuDescarrega(pAplicacio IN VARCHAR, 
                            pVersio IN VARCHAR, pDispositiu IN NUMBER, 
                            vPreuPagat OUT NUMBER) IS

BEGIN
  
  SELECT p.preu
  INTO vPreuPagat
  FROM preu p
  WHERE p.aplicacio = pAplicacio AND p.versio = pVersio AND p.pais = 
                    (select pe.pais
                     from persona pe
                     where pe.identificador = (select u.identificador
                                            from usuari u, dispositiu_usuari du
                                            where u.identificador = du.usuari 
                                            AND du.imei = pDispositiu)); 
      
END obtenirPreuDescarrega;
/


/*
 * Mtode que crea un nou registre en la taula Descrrega.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pDispositiu} number amb l'identificador del dispositiu de usuari (imei).
 * {pPagament} varchar amb el tipus de pagament.
 */
CREATE OR REPLACE PROCEDURE crearDescarrega(pAplicacio IN VARCHAR2, 
                            pVersio IN VARCHAR2, pDispositiu IN NUMBER, 
                            pPagament IN VARCHAR2) IS

vAvui DATE;
vPreuPagat NUMBER(6,2);

BEGIN
  
  --s'obt la data actual per emmagatzemar-la com data de descarrega.
  SELECT TO_DATE(TO_CHAR(SYSDATE, 'DD-MM-YYYY'), 'DD-MM-YYYY') 
  INTO vAvui FROM DUAL;  
  
  obtenirPreuDescarrega(pAplicacio, pVersio, pDispositiu, vPreuPagat);
  
  INSERT INTO descarrega(data, aplicacio, versio, dispositiu, pagament, preu)
  VALUES (vAvui, pAplicacio, pVersio, pDispositiu, pPagament, vPreuPagat);
                
END crearDescarrega;
/


/*
 * Mtode que emmagatzema una descarrega.
 * {pAplicacio} varchar amb el nom de l'aplicaci.
 * {pVersio} varchar amb la versi.
 * {pDispositiu} number amb l'imei que identifica el dispositiu de l'usuari.
 * {pPagament} varchar amb el tipus de pagament.
 * {RSP} varchar amb un missatge que indica si l'execuci ha estat correcta o no
 * i la ra per la que no ha estat aix.
 */
CREATE OR REPLACE PROCEDURE emmagatzemarDescarrega(pAplicacio IN VARCHAR2, 
                            pVersio IN VARCHAR2, pDispositiu IN VARCHAR2, 
                            pPagament IN VARCHAR2, RSP OUT VARCHAR2) IS

vUsuari VARCHAR(9);
vActivo NUMBER(1);
vExisteixAplicacio NUMBER(1);
vActiva NUMBER(1);
vExisteixDispositiu NUMBER(1);
vExisteixPagament NUMBER(1);
usuari_desactivat EXCEPTION;
aplicacio_no_existeix EXCEPTION;
aplicacio_desactivada EXCEPTION;
dispositiu_no_existeix EXCEPTION;
pagament_no_existeix EXCEPTION;
parametro_null EXCEPTION;

BEGIN
  
  /*
   * Es comproba que no existeix cap parametre null, en cas afirmatiu, es llana
   * una excepci.
   */
  IF (pAplicacio IS NULL OR pVersio IS NULL OR pDispositiu IS NULL 
      OR pPagament IS NULL) THEN 
    RAISE parametro_null;    
  END IF;
  
  /* 
   * Es comproba que existeixi el dispositiu i que l'usuari propietari del 
   * dispositiu es trobi activat al sistema, en cas contrari llana una excepci
   * indicant que no pot fer descarregues.
   */
  
  existeixDispositiuUsuari(pDispositiu, vExisteixDispositiu);
  IF vExisteixDispositiu=0 THEN 
    RAISE dispositiu_no_existeix;  
  END IF;
  
  SELECT usuari
  INTO vUsuari
  FROM dispositiu_usuari
  WHERE imei = pDispositiu;
  
  personaActivada(vUsuari, vActivo);
  IF vActivo = 0 THEN
    RAISE usuari_desactivat;
  END IF;
 
  /*
   * Es comproba si existeix la aplicaci versionada i aquesta es troba activa:
   * Si existeix --> es comproba si existeix el mode de pagament.
   *                 Si existeix --> s'emmagatzema la descarrega.
   *                 Si NO existeix --> es llana una excepci per indicar
   *                                    que el mode de pagament s incorrecte.
   * Si NO existeix --> es llana una excepci per indicar que no es pot 
   *                    realitzar una descarrega d'una aplicaci que no existeix
   */ 
  existeixAplicacioVersionada(pAplicacio, pVersio, vExisteixAplicacio);
  
  IF vExisteixAplicacio=1 THEN
    aplicacioVersionadaActivada(pAplicacio, pVersio, vActiva);
    IF vActiva =1 THEN
      existeixPagament(pPagament, vExisteixPagament);
      IF vExisteixPagament=1 THEN
        crearDescarrega(pAplicacio, pVersio, pDispositiu, pPagament);
      ELSE
        RAISE pagament_no_existeix;
      END IF;
    ELSE
      RAISE aplicacio_desactivada;
    END IF;
  ELSE
    RAISE aplicacio_no_existeix;
  END IF;
  
  -- S'afegeix al LOG la crida al mtode amb el seu resultat.
  RSP:='OK';
  afegirLog('Emmagatzemar Descarrega', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' Dispositiu: ' || pDispositiu || 
            ' Mode de pagament: ' || pPagament, RSP);
  
  EXCEPTION
    
    /*
     * Desprs de cada excepci s'afegeix al LOG la crida al mtode amb l'error
     * que ha estat provocat.
     */    
    WHEN usuari_desactivat THEN RSP:='ERROR: USER IS DISABLED';
    afegirLog('Emmagatzemar Descarrega', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' Dispositiu: ' || pDispositiu || 
            ' Mode de pagament: ' || pPagament, RSP);
            
    WHEN aplicacio_no_existeix THEN RSP:='ERROR: APPLICATION NOT EXISTS';
    afegirLog('Emmagatzemar Descarrega', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' Dispositiu: ' || pDispositiu || 
            ' Mode de pagament: ' || pPagament, RSP);
    
    WHEN aplicacio_desactivada THEN RSP:='ERROR: APPLICATION IS DISABLED';
    afegirLog('Emmagatzemar Descarrega', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' Dispositiu: ' || pDispositiu || 
            ' Mode de pagament: ' || pPagament, RSP);
    
    WHEN dispositiu_no_existeix THEN RSP:='ERROR: DEVICE NOT EXISTS';
    afegirLog('Emmagatzemar Descarrega', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' Dispositiu: ' || pDispositiu || 
            ' Mode de pagament: ' || pPagament, RSP);
    
    WHEN pagament_no_existeix THEN RSP:='ERROR: PAYMENT METHOD NOT EXISTS';
    afegirLog('Emmagatzemar Descarrega', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' Dispositiu: ' || pDispositiu || 
            ' Mode de pagament: ' || pPagament, RSP);
            
    WHEN parametro_null THEN RSP:='ERROR: NULL PARAMETER';
    afegirLog('Emmagatzemar Descarrega', 'Aplicaci: ' || pAplicacio || 
            ' Versio: ' || pVersio || ' Dispositiu: ' || pDispositiu || 
            ' Mode de pagament: ' || pPagament, RSP);
                    
END emmagatzemarDescarrega;
/

