/*
 *                    ESTADISTIQUES RAPIDES
 */


/*
 * Trigger que actualitzar la taula d'estadstiques 1 i 2  relacionades amb les
 * descarregues (total_descarregues i total_euros)
 * Tipus AFTER INSERT FOR EACH ROW
 */ 
CREATE OR REPLACE TRIGGER trigger_descarregues_est1_2
AFTER INSERT ON descarrega
FOR EACH ROW

DECLARE 

vTotal NUMBER(1);
vPreu NUMBER(6,2);

BEGIN
  
  vPreu := :new.preu;
  
  /*
   * comprobem que existeix el registre que emmagatzema els resultats totals, en 
   * cas contrari es crea un.
   * es considera de complexitat O(1) perqu noms ha de mirar si existeix un 
   * registre.
   */
  SELECT COUNT(*) INTO vTotal FROM EST1_2_TOTALS;
  
  IF vTotal=0 THEN
    INSERT INTO EST1_2_TOTALS(total_descarregues, total_euros) VALUES (0,0);
  END IF;
  
  --s'actualitzen els valors de descarregues i consum total
  UPDATE EST1_2_TOTALS SET total_descarregues = total_descarregues+1;
  UPDATE EST1_2_TOTALS SET total_euros = total_euros + vPreu;
  
END trigger_descarregues_est1_2;
/


/*
 * Trigger que actualitzar les taules d'estadstiques relacionades amb els
 * usuaris (EST1_2_TOTALS i EST3_4_5_ANY)
 * Tipus AFTER INSERT FOR EACH ROW
 */ 
CREATE OR REPLACE TRIGGER trigger_usuaris
AFTER INSERT ON usuari
FOR EACH ROW

DECLARE 

total NUMBER(1);
vAny NUMBER(4);
vExisteixAny NUMBER(1);
vQUsuaris NUMBER (10);

BEGIN
  
  /*
   * comprobem que existeix el registre que emmagatzema els resultats totals, en 
   * cas contrari es crea un a la taula EST1_2_TOTALS
   * es considera de complaxidad O(1) perqu noms ha de mirar si existeix un 
   * registre
   */
  SELECT COUNT(*) INTO total FROM EST1_2_TOTALS;
  
  IF total=0 THEN
    INSERT INTO EST1_2_TOTALS(total_descarregues, total_euros, total_usuaris) 
    VALUES (0,0,0);
  END IF;
  
  --s'obt l'any actual segons la data del sistema
  SELECT TO_NUMBER(TO_CHAR(SYSDATE, 'YYYY')) INTO vAny FROM DUAL;
  
  /*
   * s'obt un number que indica si existeix o no alguna dada per l'any actual 
   * a la taula EST3_4_5_ANY
   * es considera de complexitat O(1) perqu noms ha de mirar si existeix un 
   * registre
   */
  SELECT COUNT(*)
  INTO vExisteixAny
  FROM EST3_4_5_ANY
  WHERE any_consultat = vAny;
  
  /*
   * si no existeix cap registre amb l'any actual, es crea un nou registre el 
   * nmero de descarregues s 0, com ve predefinit en la creaci de la taula
   * i la quantitat d'usuaris s'obte de la quantitat total d'usuaris de la taula
   * EST1_2_TOTALS
   */
  IF vExisteixAny = 0 THEN
    
    --s'obt la quantitat total d'usuaris del sistema
    SELECT total_usuaris
    INTO vQUsuaris
    FROM est1_2_totals;
    
    --es crea un nou registre amb l'any i la quantitat d'usuaris registrats     
    INSERT INTO EST3_4_5_ANY(any_consultat, usuaris, nom_best_desenv, 
    best_aplicacio, desenv_best_aplicacio)
    VALUES(vAny, vQUsuaris, 'pendent', 'pendent', 'pendent');
  
  END IF;
  
  /*
   * S'actualitzen el valor d'usuaris totals de la taula EST1_2_TOTALS 
   * i usuaris per any de la taula EST3_4_5_ANY
   */
  UPDATE EST1_2_TOTALS SET total_usuaris = total_usuaris+1;
  UPDATE EST3_4_5_ANY SET usuaris = usuaris+1 WHERE any_consultat = vAny;
  
END trigger_usuaris;
/


/*
 * Trigger que actualitzar la taula d'estadstiques 3, 4 i 5  relacionades 
 * amb les descarregues per ANY
 * Tipus AFTER INSERT FOR EACH ROW
 */
CREATE OR REPLACE TRIGGER trigger_descarregues_est3_4_5
AFTER INSERT ON descarrega
FOR EACH ROW

DECLARE

vAny NUMBER(4);
vExisteixAny NUMBER(1);
vQUsuaris NUMBER (10);
vAplicacio VARCHAR2(50);
vVersio VARCHAR2(20);
vPreu NUMBER(6,2);
TYPE t_desenvolupador IS TABLE OF desenvolupador.identificador%TYPE;
vDesenvolupadors t_desenvolupador;
vNomDesenvolupador VARCHAR(50);
vMaxDescarregues NUMBER(10);
vDescDesenvolupador NUMBER(10);
vNomsDesenvolupadors VARCHAR2(200);
vMaxRecaudat NUMBER(15);
vRecaudat NUMBER(15);

BEGIN
  
  /*
   * S'obt el valor dels camps aplicaci, versi i preu de la descarrega que
   * s'acaba de registrar.
   */
  vAplicacio := :new.aplicacio;
  vVersio := :new.versio;
  vPreu := :new.preu;
  
  --s'obt l'any actual segons la data del sistema.
  SELECT TO_NUMBER(TO_CHAR(SYSDATE, 'YYYY')) INTO vAny FROM DUAL;
  
  /*
   * s'obt un number que indica si existeix o no alguna dada per l'any actual
   * es considera de complexitat O(1) perqu noms ha de mirar si existeix un 
   * registre.
   */
  SELECT COUNT(*)
  INTO vExisteixAny
  FROM EST3_4_5_ANY
  WHERE any_consultat = vAny;
  
  /*
   * si no existeix cap registre amb l'any actual, es crea un nou registre el 
   * nmero de descarregues s 0, com ve predefinit en la creaci de la taula.
   */
  IF vExisteixAny = 0 THEN
    
    --s'obt la quantitat total d'usuaris del sistema.
    SELECT total_usuaris
    INTO vQUsuaris
    FROM est1_2_totals;
    
    --es crea un nou registre amb l'any i la quantitat d'usuaris registrats.     
    INSERT INTO EST3_4_5_ANY(any_consultat, usuaris, nom_best_desenv, 
    best_aplicacio, desenv_best_aplicacio)
    VALUES(vAny, vQUsuaris, 'pendent', 'pendent', 'pendent');
    
    /*
     * com es tracta d'un any nou, es reinicien tots els valors de euros 
     * generats per totes les aplicacions i els valors de descarregues prpies
     * de tots els desenvolupadors.
     */
    UPDATE aplicacio_versionada SET euros_generats=0;
    UPDATE desenvolupador SET descarregues_app_propies=0;
  
  END IF;
  
  /*
   * s'obtenen el desenvolupador o desenvolupadors de l'aplicaci que s'ha 
   * descarregat.
   */
  SELECT  da.desenvolupador
  BULK COLLECT INTO vDesenvolupadors
  FROM desenvolupador_aplicacio da 
  WHERE da.aplicacio=vAplicacio AND da.versio=vVersio;
  
  -- es realitzen una serie de operacions per cada desenvolupador.
  FOR i IN vDesenvolupadors.FIRST .. vDesenvolupadors.LAST LOOP
    
    --s'obte el nom del desenvolupador.
    SELECT nom
    INTO vNomDesenvolupador
    FROM persona
    WHERE identificador = vDesenvolupadors(i);
    
    /* 
     * s'emmagatzema en nom del desenvolupador en una variable per poder tenir 
     * tots els noms dels desenvolupadors de l'aplicaci descarregada.
     */
    vNomsDesenvolupadors := vNomsDesenvolupadors || vnomdesenvolupador || '. ';
    
    --s'obte l'actual mxim de descarregues per desenvolupador.
    SELECT desc_best_desenv 
    INTO vMaxDescarregues
    FROM est3_4_5_any 
    WHERE any_consultat=vAny;
    
    /*
     * s'actulitza la quantitat de descarregues prpies del desenvolupador de 
     * l'aplicaci descarregada.
     */
    UPDATE desenvolupador 
    SET descarregues_app_propies = descarregues_app_propies+1
    WHERE identificador = vDesenvolupadors(i)
    RETURNING descarregues_app_propies
    INTO vDescDesenvolupador;
    
    /* 
     * si la quantitat de descarregues del desenvolupador es superior al mxim, 
     * s'actulitza el mxim i el nom del desenvolupador amb el mxim de 
     * descarregues.
     */
    IF vDescDesenvolupador > vMaxDescarregues THEN
      UPDATE est3_4_5_any
      SET desc_best_desenv = vDescDesenvolupador, 
          nom_best_desenv =  vNomDesenvolupador
      WHERE any_consultat = vAny;
    END IF;
    
    /* 
     * si la quantitat de descarregues del desenvolupador es igual al mxim, 
     * s'afegeix el nom del desenvolupdor al camp nom_best_desenv que 
     * emmagatzema els noms dels desenvolupadors amb ms descarrgues de les 
     * seves aplicacions.
     */
    IF vDescDesenvolupador = vMaxDescarregues THEN
      UPDATE est3_4_5_any
      SET nom_best_desenv =  nom_best_desenv || '. ' || vNomDesenvolupador
      WHERE any_consultat = vAny;
    END IF;  
    
  END LOOP;
  
  --s'obte l'actual mxim de euros generats per una aplicaci.
  SELECT euros_best_aplicacio
  INTO vMaxRecaudat
  FROM est3_4_5_any
  WHERE any_consultat = vAny;
  
  /*
   * s'actulitza la quantitat d'euros generats per la descarrega de l'aplicaci 
   * descarregada.
   */
  UPDATE aplicacio_versionada
  SET euros_generats = euros_generats + vPreu 
  WHERE aplicacio = vAplicacio AND versio = vVersio
  RETURNING euros_generats
  INTO vRecaudat;
  
  /* 
   * si la quantitat d'euros generats per l'aplicaci es superior al mxim, 
   * s'actulitza el mxim, el nom i la versi de l'aplicaci que ha generat el 
   * mxim d'euros i el nom del desenvolupador o desenvolupadors de l'aplicaci.
   */
  IF vRecaudat > vMaxRecaudat THEN 
    UPDATE est3_4_5_any
      SET euros_best_aplicacio = vRecaudat,
          best_aplicacio = vAplicacio || ' ' || vVersio || '. ',
          desenv_best_aplicacio = vNomsDesenvolupadors
      WHERE any_consultat = vAny;  
  END IF;
  
  /* 
   * si la quantitat d'euros generats per l'aplicaci es igual al mxim, 
   * s'afegeix el nom de l'aplicaci i la versi al camp best_aplicaci que 
   * emmagatzema les aplicacions amb el mxim recaudat i tamb s'afegeix el nom
   * o noms del desenvolupador o desenvolupadors en el camp 
   * desenv_best_aplicaci que emmagatzema el noms dels desenvolupadors de les 
   * aplicacions que ms euros han generat.
   */
  IF vRecaudat = vMaxRecaudat THEN    
    UPDATE est3_4_5_any
    SET best_aplicacio = best_aplicacio || vAplicacio || ' ' || vVersio || '. ',
    desenv_best_aplicacio = desenv_best_aplicacio || '. ' || vNomsDesenvolupadors
    WHERE any_consultat = vAny;  
  END IF;
    
  /*
   * s'actulitza el nmero de descarregues durant l'any actual.
   */
  UPDATE EST3_4_5_ANY 
  SET descarregues = descarregues+1
  WHERE any_consultat = vAny;
         
END trigger_descarregues_est3_4_5;
/


/*
 * Trigger que actualitzar el valor de la mitjana de aplicacions descarregades
 * per usuari
 * Tipus AFTER UPDATE ON EST3_4_5_ANY
 */
 
CREATE OR REPLACE TRIGGER trigger_mitjana
AFTER UPDATE OF descarregues, usuaris ON est3_4_5_any

DECLARE 

vAny NUMBER(4);

BEGIN
  
   --s'obt l'any actual segons la data del sistema
  SELECT TO_NUMBER(TO_CHAR(SYSDATE, 'YYYY')) INTO vAny FROM DUAL;
  
  --es calcula la mitjana i s'actualitza el seu valor.
  UPDATE EST3_4_5_ANY 
  SET mitjana_desc_user = descarregues/usuaris 
  WHERE any_consultat=vAny;
  
END trigger_mitjana;
/


/*
 * Trigger que actualitzar les estadstiques 6, 7 i 8 relacionades amb les 
 * descarregues per ANY i PAIS de la taula EST6_7_8_ANY_PAIS
 * Tipus BEFORE INSERT FOR EACH ROW
 */
CREATE OR REPLACE TRIGGER trigger_descarregues_est6_7_8
AFTER INSERT ON descarrega
FOR EACH ROW

DECLARE

vImei NUMBER(15);
vPreu NUMBER(10,2);
vAplicacio VARCHAR2(50);
vVersio VARCHAR2(20);
vAny NUMBER(4);
vNomPais VARCHAR2(50);
vUsuari VARCHAR2(9);
vPais VARCHAR2(2);
vExisteixAny NUMBER(3);
vExisteixAnyPais NUMBER(1);
vUsuariDescarregat VARCHAR2(5);
vPaisosAplicacioDescarregada VARCHAR2(50);
vExisteixPais NUMBER(2);

BEGIN
  
  /*
   * S'obt el valor dels camps dispositiu, preu, aplicaci i versi de la 
   * descarrega que s'acaba de registrar.
   */
  vImei := :new.dispositiu;
  vPreu := :new.preu;
  vAplicacio := :new.aplicacio;
  vVersio := :new.versio;
  
  --s'obt l'any actual segons la data del sistema
  SELECT TO_NUMBER(TO_CHAR(SYSDATE, 'YYYY')) INTO vAny FROM DUAL;
  
  /*
   * s'obte el pais on s'ha realitzat la descarrega i l'usuari que al que 
   * pertany el dispositiu on s'ha realitzat.
   */
  obtenirDadesDispositiu(vImei, vNomPais, vUsuari);
  
  --s'obte l'identificador del pais de l'usuari propietari del dispositiu
  obtenirIdPais(vNomPais, vPais);
  
  /*
   * s'obt un number que indica si existeix o no alguna dada per l'any actual 
   * a la taula EST6_7_8_ANY_PAIS
   * es considera de complexitat O(1) perqu noms ha de mirar si existeix un 
   * registre
   */
  SELECT COUNT(*)
  INTO vExisteixAny
  FROM EST6_7_8_ANY_PAIS
  WHERE any_consultat = vAny;
  
  /*
   * si no existeix cap registre amb l'any actual, es reinicien tots els valors
   * de primera descarrega de tots els usuaris a 'false' i la llista de paisos
   * on s'ha descarregat una aplicaci es buida.
   */  
  IF vExisteixAny = 0 THEN
    UPDATE usuari SET primera_descarrega = 'false';
    UPDATE aplicacio_versionada SET paisos_descarregada = ' ';
  END IF;
  
  /* 
   * s'obt un number que indica si existeix o no alguna dada per l'any actual i
   * el pais de la descarrega.
   * es considera de complexitat O(1) perqu noms ha de mirar si existeix un 
   * registre.
   */
  SELECT COUNT(*)
  INTO vExisteixAnyPais
  FROM EST6_7_8_ANY_PAIS
  WHERE any_consultat = vAny AND pais = vNomPais;
  
  /*
   * si no existeix cap registre amb l'any actual i el pais, es crea un nou 
   * registre on usuaris_1_desc, aplicacions_1_desc y total_euros s 0, com ve 
   * predefinit en la creaci de la taula.
   */
  IF vExisteixAnyPais = 0 THEN
    --es crea un nou registre amb l'any i el pais del dispositiu.     
    INSERT INTO EST6_7_8_ANY_PAIS(any_consultat, pais)
    VALUES(vAny, vNomPais);
  END IF;
  
  /* 
   * s'obte el valor de primera_descarrega de l'usuari que ha relitzat la 
   * descarrega, per saber si ja ha realitzat alguna durant aquest any.
   */
  SELECT primera_descarrega
  INTO vUsuariDescarregat
  FROM usuari
  WHERE identificador = vUsuari;
  
  /* 
   * si el valor s 'false', s'actualitza a 'true' per indicar que ja ha 
   * realitzat la seva primera descarrega durant aquest any i s'actulitza la 
   * quantitat d'usuaris amb una descarrega o ms en l'any actual i el pais de 
   * l'usuari. 
   */
  IF vUsuariDescarregat='false' THEN
    UPDATE usuari SET primera_descarrega='true' WHERE identificador=vUsuari;
    UPDATE est6_7_8_any_pais SET usuaris_1_desc=usuaris_1_desc+1 
                             WHERE any_consultat=vAny AND pais=vNomPais;
  END IF;
  
  /* 
   * s'obte el llistat de codis on s'ha descarregat l'aplicaci recentment 
   * descarregada.
   */
  SELECT paisos_descarregada
  INTO vPaisosAplicacioDescarregada
  FROM aplicacio_versionada
  WHERE aplicacio = vAplicacio AND versio = vVersio;
  
  -- es comproba si s'ha descarregat ja algun cop en el pais de l'usuari
  SELECT INSTR(vPaisosAplicacioDescarregada,vPais,1,1) 
  INTO vExisteixPais 
  FROM DUAL;
  
  /*
   * si no s'ha descarregat cap cop en aquest pas, s'actulitza la llista de 
   * paisos on s'ha descarregat l'aplicaci inserint el codi del pas de 
   * l'usuari i s'atualitza el nmero d'aplicacions amb un mnim d'una 
   * descarrega durant l'any actual en el pas de l'usuari.
   */
  IF vExisteixPais=0 THEN
  
    UPDATE aplicacio_versionada 
    SET paisos_descarregada = paisos_descarregada || vPais || ' '
    WHERE aplicacio=vAplicacio AND versio=vVersio;
    
    UPDATE est6_7_8_any_pais 
    SET aplicacions_1_desc = aplicacions_1_desc+1
    WHERE any_consultat=vAny AND pais=vNomPais;
    
  END IF;
  
  /*
   * s'actualitza el total d'euros generat durant l'any actual en el pais de 
   * l'usuari sumant el preu de la descarrega.
   */
  UPDATE est6_7_8_any_pais 
  SET total_euros = total_euros + vPreu
  WHERE any_consultat=vAny AND pais=vNomPais;
         
END trigger_descarregues_est6_7_8;
/

