/*
 *        METODES DE CONSULTA
 */


/*
 *      CONSULTA A
 */
 
/*
 * Es crea un nou tipus de objecte amb l'estructura que tindr un registre de la
 * taula que retorni la consulta
 */
create or replace TYPE TYPE_CONSULTA_A AS OBJECT (
  dni VARCHAR2(9), 
  nom VARCHAR2(50),
  adreca VARCHAR2(100),
	telefon NUMBER(9),
	mail VARCHAR2(50),
  activo VARCHAR2(5),
  representant VARCHAR2(50),
  total_aplicacions NUMBER(5)
);
/

/*
 * Es crea un nou tipus de taula que contindr els objectes anteriors
 */
create or replace TYPE TAULA_CONSULTA_A AS TABLE OF TYPE_CONSULTA_A;             
/

/*
 * Funci que retorna un llistat amb tots els desenvolupadors d'un pais amb 
 * totes les seves dades i el nmero d'aplicacions publicades
 * {pPais} varchar amb el nom del pas
 * retorna una taula TAULA_CONSULTA_A
 */
CREATE OR REPLACE FUNCTION dadesDesenvolupadorPais(pPais IN VARCHAR2) 
                                                      RETURN TAULA_CONSULTA_A IS
  
  vDni VARCHAR2(9); 
  vNom VARCHAR2(50);
  vAdreca VARCHAR2(100);
	vTelefon NUMBER(9);
	vMail VARCHAR2(50);
  vActivo VARCHAR2(5);
  vRepresentant VARCHAR2(50);
  vTotal_aplicacions NUMBER(5);
  resultat TAULA_CONSULTA_A:=TAULA_CONSULTA_A();
  
  TYPE r_cursor IS REF CURSOR;
  vCursor r_cursor;
  i integer:=1;
  
  vExisteixPais NUMBER(1);
  vIdPais VARCHAR2(2);
  RSP VARCHAR2(200);
  pais_no_existeix EXCEPTION;
  parametro_null EXCEPTION;

BEGIN
  
  /*
   * Es comproba que el parametre pPais no s null, en cas afirmatiu, es llana
   * una excepci
   */
  IF (pPais IS NULL) THEN 
    RAISE parametro_null;    
  END IF;
  
  /*
   * Es comproba que existeix un pas amb aquest nom, en cas contrari, es llana
   * una excepci
   */
  existeixPais(pPais, vExisteixPais);
  
  IF vExisteixPais = 0 THEN
    RAISE pais_no_existeix;
  END IF;
  
  --s'obte l'identificador del pas
  obtenirIdPais(pPais, vIdPais);
  
  --es relitza la consulta
  OPEN vCursor FOR    
    SELECT p.identificador, p.nom, p.adreca, p.telefon, p.mail, p.activo, 
           d.representant, COUNT(da.desenvolupador)
    FROM persona p INNER JOIN desenvolupador d
    ON p.pais=vIdPais AND p.identificador = d.identificador
    INNER JOIN desenvolupador_aplicacio da
    ON p.identificador = da.desenvolupador
    GROUP BY p.identificador, p.nom, p.adreca, p.telefon, p.mail, p.activo, 
             d.representant;
  
    LOOP
      FETCH vCursor 
      INTO vDni, vNom, vAdreca, vTelefon, vMail, vActivo, vRepresentant, 
           vTotal_aplicacions;
      EXIT WHEN vCursor %NOTFOUND;
        resultat.extend;
        resultat(i):= TYPE_CONSULTA_A(vDni, vNom, vAdreca, vTelefon, vMail, 
                                    vActivo, vRepresentant, vTotal_aplicacions);
        i:=i+1;
    END LOOP;
  CLOSE vCursor;
  
  -- S'afegeix al LOG la crida a la funci amb el seu resultat
  RSP:='OK';
  afegirLog('Consulta dades desenvolupadors per pas', 'Pas: ' || pPais, RSP);
  
  RETURN resultat;
  
  EXCEPTION
  
  /*
   * Desprs de cada excepci s'afegeix al LOG la crida a la funci amb l'error
   * que ha estat provocat
   */
  WHEN parametro_null THEN RSP:='ERROR: NULL PARAMETER';
    afegirLog('Consulta dades desenvolupadors per pas', 'Pas: ' || pPais,RSP);
    RETURN NULL;
            
  WHEN pais_no_existeix THEN RSP:='ERROR: COUNTRY NOT EXISTS';
    afegirLog('Consulta dades desenvolupadors per pas', 'Pas: ' || pPais,RSP);
    RETURN NULL;
     
END dadesDesenvolupadorPais;
/


/*
 *      CONSULTA B
 */

/*
 * Es crea un nou tipus de objecte amb l'estructura que tindr un registre de la
 * taula que retorni la consulta
 */
create or replace TYPE TYPE_CONSULTA_B AS OBJECT (
  aplicacio VARCHAR2(50), 
  versio VARCHAR2(20),
  resolucio VARCHAR2(20),
	url VARCHAR2(100),
  data_pujada DATE,
  total_descarregues NUMBER(5)
);
/

/*
 * Es crea un nou tipus de taula que contindr els objectes anteriors
 */
create or replace TYPE TAULA_CONSULTA_B AS TABLE OF TYPE_CONSULTA_B;             
/

/*
 * Funci que retorna un llistat amb totes les aplicacions actives i les seves 
 * dades, ordenades pel nmero total de descrregues
 * retorna una taula TAULA_CONSULTA_B
 */
CREATE OR REPLACE FUNCTION dadesAplicacionsActives RETURN TAULA_CONSULTA_B IS
  
  vAplicacio VARCHAR2(50);
  vVersio VARCHAR2(20);
  vResolucio VARCHAR2(20);
	vUrl VARCHAR2(100);
  vData_pujada DATE;
  vTotal_descarregues NUMBER(5);
  resultat TAULA_CONSULTA_B:=TAULA_CONSULTA_B();
  
  TYPE r_cursor IS REF CURSOR;
  vCursor r_cursor;
  i integer:=1;
  
  RSP VARCHAR2(200);

BEGIN
  
  --es relitza la consulta
  OPEN vCursor FOR    
    SELECT ap.aplicacio, ap.versio, r.resolucio, ap.url, ap.data_pujada, 
           COUNT(d.aplicacio)
    FROM aplicacio_versionada ap INNER JOIN resolucio r
    ON ap.resolucio_minima = r.identificador AND ap.activa='true' 
    LEFT OUTER JOIN descarrega d
    ON ap.aplicacio = d.aplicacio AND ap.versio = d.versio
    GROUP BY ap.aplicacio, ap.versio, r.resolucio, ap.url, ap.data_pujada
    ORDER BY COUNT (d.aplicacio) DESC;
  
    LOOP
      FETCH vCursor 
      INTO vAplicacio, vVersio, vResolucio, vUrl, vData_pujada, 
           vTotal_descarregues;      
      EXIT WHEN vCursor %NOTFOUND;
        resultat.extend;
        resultat(i):= TYPE_CONSULTA_B(vAplicacio, vVersio, vResolucio, vUrl, 
                                             vData_pujada, vTotal_descarregues);
        i:=i+1;
    END LOOP;
  CLOSE vCursor;
  
  -- S'afegeix al LOG la crida a la funci amb el seu resultat
  RSP:='OK';
  afegirLog('Consulta dades aplicacions actives', 'Sense parmetres', RSP);
  
  RETURN resultat;
     
END dadesAplicacionsActives;
/


/*
 *      CONSULTA C
 */

/*
 * Es crea un nou tipus de objecte amb l'estructura que tindr un registre de la
 * taula que retorni la consulta
 */
create or replace TYPE TYPE_CONSULTA_C AS OBJECT (
  pais VARCHAR2(50),
  descarregues_pais NUMBER(5)
);
/

/*
 * Es crea un nou tipus de taula que contindr els objectes anteriors
 */
create or replace TYPE TAULA_CONSULTA_C AS TABLE OF TYPE_CONSULTA_C;             
/

/*
 * Funci que retorna un llistat de tots els pasos on s'ha descarregat una 
 * aplicacio determinada durant un any determinat, aix com el nmero de 
 * descarregues
 * retorna una taula TAULA_CONSULTA_C
 */
CREATE OR REPLACE FUNCTION paisosAplicacioAny (pAplicacio IN VARCHAR2, 
                           pVersio IN VARCHAR2, pAny IN VARCHAR2) 
                          RETURN TAULA_CONSULTA_C IS

  vPais VARCHAR2(50);
  vDescarregues_pais NUMBER(5);
  resultat TAULA_CONSULTA_C:=TAULA_CONSULTA_C();
  
  TYPE r_cursor IS REF CURSOR;
  vCursor r_cursor;
  i integer:=1;
  
  RSP VARCHAR2(200);
  vExisteixAplicacio NUMBER(1);
  aplicacio_no_existeix EXCEPTION;
  parametro_null EXCEPTION;

BEGIN
  
  /*
   * Es comproba que no existeix cap parametre null, en cas afirmatiu, es llana
   * una excepci
   */
  IF (pAplicacio IS NULL OR pVersio IS NULL OR pAny IS NULL) THEN 
    RAISE parametro_null;    
  END IF;
  
  /*
   * Es comproba que existeix l'aplicaci versionada, en cas contrari, es llana
   * una excepci indicant que no es pot coneixer els paisos on s'ha descarregat 
   * una aplicaci que no existeix
   */
  existeixAplicacioVersionada(pAplicacio, pVersio, vExisteixAplicacio);
  
  IF vExisteixAplicacio = 0 THEN
    RAISE aplicacio_no_existeix;
  END IF;
  
  --es relitza la consulta
  OPEN vCursor FOR    
    SELECT pa.pais, COUNT(pa.pais)
    FROM pais pa INNER JOIN persona pe
    ON pa.identificador = pe.pais
    INNER JOIN usuari u 
    ON pe.identificador = u.identificador
    INNER JOIN dispositiu_usuari du 
    on u.identificador = du.usuari
    INNER JOIN descarrega d
    ON d.aplicacio = pAplicacio AND d.versio = pVersio 
                    AND d.dispositiu = du.imei AND TO_CHAR(d.data,'YYYY') = pAny
    GROUP BY pa.pais
    ORDER BY COUNT(pa.pais) DESC;
  
    LOOP
      FETCH vCursor INTO vPais, vDescarregues_pais;      
      EXIT WHEN vCursor %NOTFOUND;
        resultat.extend;
        resultat(i):= TYPE_CONSULTA_C(vPais, vDescarregues_pais);
        i:=i+1;
    END LOOP;
  CLOSE vCursor;
  
  -- S'afegeix al LOG la crida a la funci amb el seu resultat
  RSP:='OK';
  afegirLog('Consulta paisos segons aplicaci i any', 'Aplicacio: ' || 
            pAplicacio || ' Versi: ' || pVersio || ' Any: ' || pAny, RSP);
  
  RETURN resultat;
  
  EXCEPTION
  
  /*
   * Desprs de cada excepci s'afegeix al LOG la crida a la funci amb l'error
   * que ha estat provocat
   */
  WHEN parametro_null THEN RSP:='ERROR: NULL PARAMETER';
    afegirLog('Consulta paisos segons aplicaci i any', 'Aplicacio: ' || 
              pAplicacio || ' Versi: ' || pVersio || ' Any: ' || pAny, RSP);
    RETURN NULL;
            
  WHEN aplicacio_no_existeix THEN RSP:='ERROR: APPLICATION NOT EXISTS';
    afegirLog('Consulta paisos segons aplicaci i any', 'Aplicacio: ' || 
              pAplicacio || ' Versi: ' || pVersio || ' Any: ' || pAny, RSP);
    RETURN NULL;
     
END paisosAplicacioAny;
/
  

/*
 *      CONSULTA D
 */

/*
 * Es crea un nou tipus de objecte amb l'estructura que tindr un registre de la
 * taula que retorni la consulta
 */
 
create or replace TYPE TYPE_CONSULTA_D AS OBJECT (
  nom VARCHAR2(50),
  id_descarrega NUMBER(20),
  data DATE,
  dispositiu  NUMBER(15),
  aplicacio VARCHAR(50),
  versio VARCHAR(20),    
  pagament VARCHAR(50),
  preu NUMBER(6,2)  
);
/

/*
 * Es crea un nou tipus de taula que contindr els objectes anteriors
 */
create or replace TYPE TAULA_CONSULTA_D AS TABLE OF TYPE_CONSULTA_D;             
/

/*
 * Funci que retorna un llistat amb l'histric de descrregues d'un usuari,
 * identificat pel seu nmero de telfon, en tots els seus dispositius associats
 * retorna una taula TAULA_CONSULTA_D
 */
CREATE OR REPLACE FUNCTION activitatUsuari (pTelefon IN NUMBER) 
                                                      RETURN TAULA_CONSULTA_D IS

  vNom VARCHAR2(50);
  vId_descarrega NUMBER(20);
  vData DATE;
  vDispositiu  NUMBER(15);
  vAplicacio VARCHAR(50);
  vVersio VARCHAR(20); 
  vPagament VARCHAR(50);
  vPreu NUMBER(6,2);
  resultat TAULA_CONSULTA_D:=TAULA_CONSULTA_D();
  
  TYPE r_cursor IS REF CURSOR;
  vCursor r_cursor;
  i integer:=1;
  
  RSP VARCHAR2(200);
  parametro_null EXCEPTION;

BEGIN
  
  /*
   * Es comproba que el parametre pTelefon no s null, en cas afirmatiu, es 
   * llana una excepci
   */
  IF (pTelefon IS NULL) THEN 
    RAISE parametro_null;    
  END IF;
  
  --es relitza la consulta
  OPEN vCursor FOR    
    SELECT p.nom, d.identificador, d.data, d.dispositiu, d.aplicacio, d.versio, 
           d.pagament, d.preu 
    FROM descarrega d INNER JOIN dispositiu_usuari du
    ON d.dispositiu = du.imei
    INNER JOIN persona p
    ON du.usuari = p.identificador AND p.telefon = pTelefon
    ORDER BY d.identificador;
  
    LOOP
      FETCH vCursor 
      INTO vNom, vId_descarrega, vData, vDispositiu, vAplicacio, vVersio, 
           vPagament, vPreu;      
      EXIT WHEN vCursor %NOTFOUND;
        resultat.extend;
        resultat(i):= TYPE_CONSULTA_D(vNom, vId_descarrega, vData, vDispositiu, 
                                         vAplicacio, vVersio, vPagament, vPreu);
        i:=i+1;
    END LOOP;
  CLOSE vCursor;
  
  -- S'afegeix al LOG la crida a la funci amb el seu resultat
  RSP:='OK';
  afegirLog('Consulta activitat usuari', 'Telfon: ' || pTelefon, RSP);
  
  RETURN resultat;
  
  EXCEPTION
  
  /*
   * Desprs de cada excepci s'afegeix al LOG la crida a la funci amb l'error
   * que ha estat provocat
   */
  WHEN parametro_null THEN RSP:='ERROR: NULL PARAMETER';
    afegirLog('Consulta activitat usuari', 'Telfon: ' || pTelefon, RSP);
    RETURN NULL;
     
END activitatUsuari;
/


/*
 *      CONSULTA E
 */

/*
 * Es crea un nou tipus de objecte amb l'estructura que tindr un registre de la
 * taula que retorni la consulta
 */
 
create or replace TYPE TYPE_CONSULTA_E AS OBJECT (
  nom VARCHAR2(50),
  dni VARCHAR2(9),
  pais VARCHAR2(50),
  consum NUMBER(10,2)  
);
/

/*
 * Es crea un nou tipus de taula que contindr els objectes anteriors
 */
create or replace TYPE TAULA_CONSULTA_E AS TABLE OF TYPE_CONSULTA_E;             
/

/*
 * Funci que retorna un llistat amb els 20 millors usuaris amb el seu consum en
 * un any determinat
 * retorna una taula TAULA_CONSULTA_E
 */
CREATE OR REPLACE FUNCTION top20ConsumAny(pAny IN VARCHAR2) 
                                                      RETURN TAULA_CONSULTA_E IS

  vNom VARCHAR2(50);
  vDni VARCHAR2(9);
  vPais VARCHAR2(50);
  vConsum NUMBER(6,2);
  resultat TAULA_CONSULTA_E:=TAULA_CONSULTA_E();
  
  TYPE r_cursor IS REF CURSOR;
  vCursor r_cursor;
  i integer:=1;
  
  RSP VARCHAR2(200);
  parametro_null EXCEPTION;

BEGIN
  
  /*
   * Es comproba que el parametre pAny no s null, en cas afirmatiu, es 
   * llana una excepci
   */
  IF (pAny IS NULL) THEN 
    RAISE parametro_null;    
  END IF;
  
  --es relitza la consulta
  OPEN vCursor FOR        
    SELECT nom, DNI, pais, gasto FROM (
        SELECT p.nom AS nom, p.identificador AS DNI, pa.pais AS pais, 
               SUM(d.preu) AS gasto
        FROM persona p INNER JOIN pais pa
        ON p.pais = pa.identificador
        INNER JOIN dispositiu_usuari du
        ON p.identificador = du.usuari
        INNER JOIN descarrega d
        ON du.imei = d.dispositiu AND TO_CHAR(d.data,'YYYY') = pAny
        GROUP BY p.nom, p.identificador, pa.pais
        ORDER BY SUM(d.preu) DESC, p.identificador)
    WHERE ROWNUM <= 20;
  
    LOOP
      FETCH vCursor INTO vNom, vDni, vPais, vConsum;      
      EXIT WHEN vCursor %NOTFOUND;
        resultat.extend;
        resultat(i):= TYPE_CONSULTA_E(vNom, vDni, vPais, vConsum);
        i:=i+1;
    END LOOP;
  CLOSE vCursor;
  
  -- S'afegeix al LOG la crida a la funci amb el seu resultat
  RSP:='OK';
  afegirLog('Consulta TOP 20 usuaris-consum-any', 'Any: ' || pAny, RSP);
  
  RETURN resultat;
  
  EXCEPTION
  
  /*
   * Desprs de cada excepci s'afegeix al LOG la crida a la funci amb l'error
   * que ha estat provocat
   */
  WHEN parametro_null THEN RSP:='ERROR: NULL PARAMETER';
    afegirLog('Consulta TOP 20 usuaris-consum-any', 'Any: ' || pAny, RSP);
    RETURN NULL;
     
END top20ConsumAny;
/
